"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_js_1 = require("@deep-foundation/hasura/client.js");
const benchmark_1 = require("benchmark");
const client_js_2 = require("../imports/client.js");
const lodash_1 = __importDefault(require("lodash"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:benchmarks');
const log = debug.extend('log');
const error = debug.extend('error');
// Force enable this file errors output
const delay = time => new Promise(res => setTimeout(res, time));
const apolloClient = (0, client_js_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({ apolloClient });
(async () => {
    var suite = new benchmark_1.Suite();
    const admin = await deep.jwt({ linkId: await deep.id('deep', 'admin') });
    const deepAdmin = new client_js_2.DeepClient({ deep: deep, token: admin.token, linkId: admin.linkId });
    const Query = await deep.id('@deep-foundation/core', 'Query');
    const guest = await deep.guest({});
    const deepGuest = new client_js_2.DeepClient({ deep: deepAdmin, ...guest });
    await deepAdmin.insert({
        type_id: await deep.id('@deep-foundation/core', 'Rule'),
        out: { data: [
                {
                    type_id: await deep.id('@deep-foundation/core', 'RuleSubject'),
                    to: { data: {
                            type_id: await deep.id('@deep-foundation/core', 'Selector'),
                            out: { data: [
                                    {
                                        type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                                        to_id: guest.linkId,
                                        out: { data: {
                                                type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                                                to_id: await deep.id('@deep-foundation/core', 'containTree'),
                                            } },
                                    },
                                ] }
                        } }
                },
                {
                    type_id: await deep.id('@deep-foundation/core', 'RuleObject'),
                    to: { data: {
                            type_id: await deep.id('@deep-foundation/core', 'Selector'),
                            out: { data: {
                                    type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                                    to_id: await deep.id('@deep-foundation/core'),
                                    out: { data: {
                                            type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                                            to_id: await deep.id('@deep-foundation/core', 'containTree'),
                                        } },
                                } }
                        } }
                },
                {
                    type_id: await deep.id('@deep-foundation/core', 'RuleAction'),
                    to: { data: {
                            type_id: await deep.id('@deep-foundation/core', 'Selector'),
                            out: { data: {
                                    type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                                    to_id: await deep.id('@deep-foundation/core', 'AllowInsertType'),
                                    out: { data: {
                                            type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                                            to_id: await deep.id('@deep-foundation/core', 'containTree'),
                                        } },
                                } }
                        } }
                },
            ] },
    });
    await (new Promise((res) => {
        suite.add('3000', { defer: true, fn: async function (deferred) {
                await delay(3000);
                deferred.resolve();
            } });
        suite.add('by deepRoot.id', { defer: true, fn: async function (deferred) {
                await deep.id('@deep-foundation/core', 'Promise');
                deferred.resolve();
            } });
        // suite.add('by deepGuest.id', { defer: true, fn: async function(deferred) {
        //   await deepGuest.id('@deep-foundation/core', 'Promise');
        //   deferred.resolve();
        // } });
        suite.add('deepAdmin.insert { type: Any } x1/1tr', { defer: true, fn: async function (deferred) {
                await deepAdmin.insert({ type_id: Query });
                deferred.resolve();
            } });
        suite.add('deepAdmin.insert { type: Any } x100/1tr', { defer: true, fn: async function (deferred) {
                await deepAdmin.insert(lodash_1.default.times(100, (t) => ({ type_id: Query })));
                deferred.resolve();
            } });
        suite.add('deepAdmin.insert { type: Any } x1000/1tr', { defer: true, fn: async function (deferred) {
                await deepAdmin.insert(lodash_1.default.times(1000, (t) => ({ type_id: Query })));
                deferred.resolve();
            } });
        // suite.add('deepGuest.insert { type: Any } x1/1tr', { defer: true, fn: async function(deferred) {
        //   await deepGuest.insert({ type_id: Query });
        //   deferred.resolve();
        // } });
        // suite.add('deepGuest.insert { type: Any } x100/1tr', { defer: true, fn: async function(deferred) {
        //   await deepGuest.insert(_.times(100, (t) => ({ type_id: Query })));
        //   deferred.resolve();
        // } });
        // suite.add('deepGuest.insert { type: Any } x1000/1tr', { defer: true, fn: async function(deferred) {
        //   await deepGuest.insert(_.times(1000, (t) => ({ type_id: Query })));
        //   deferred.resolve();
        // } });
        suite.on('cycle', function (event) {
            log(String(event.target));
        });
        suite.on('complete', function () {
            log('Fastest is ' + this.filter('fastest').map('name'));
            res(undefined);
        });
        // run async
        suite.run({ 'async': false });
    }));
})();
//# sourceMappingURL=index.js.map