"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.boolExpToSQL = exports.applyBoolExpToLink = exports.userPublicSymbol = exports.itemPublicSymbol = exports.userReplaceSymbol = exports.itemReplaceSymbol = exports.api = void 0;
const debug_1 = __importDefault(require("debug"));
const client_js_1 = require("@deep-foundation/hasura/client.js");
const api_js_1 = require("@deep-foundation/hasura/api.js");
const client_js_2 = require("./client.js");
const debug = (0, debug_1.default)('deeplinks:bool_exp');
const log = debug.extend('log');
const error = debug.extend('error');
// Force enable this file errors output
exports.api = new api_js_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({
    apolloClient: client,
});
exports.itemReplaceSymbol = 777777777777;
exports.userReplaceSymbol = 777777777778;
exports.itemPublicSymbol = 'X-Deep-Item-Id';
exports.userPublicSymbol = 'X-Deep-User-Id';
const applyBoolExpToLink = (sql, linkId) => {
    return sql.replace(`${exports.itemReplaceSymbol}`, `${linkId}`);
};
exports.applyBoolExpToLink = applyBoolExpToLink;
const boolExpToSQL = async (boolExpId, boolExpValue) => {
    log('boolExpToSQL', boolExpId, boolExpValue);
    let gql, explained, sql;
    try {
        if (typeof (boolExpValue) !== 'object') {
            throw new Error('boolExpValue must be object');
        }
        const serializedQuery = deep.serializeQuery(boolExpValue);
        gql = JSON.stringify(serializedQuery?.where).replace(/"([^"]+)":/g, '$1:');
        gql = gql.replace(new RegExp(`'${exports.userPublicSymbol}'`, 'g'), exports.userReplaceSymbol);
        gql = gql.replace(new RegExp(`"${exports.userPublicSymbol}"`, 'g'), exports.userReplaceSymbol);
        gql = gql.replace(new RegExp(`'${exports.itemPublicSymbol}'`, 'g'), exports.itemReplaceSymbol);
        gql = gql.replace(new RegExp(`"${exports.itemPublicSymbol}"`, 'g'), exports.itemReplaceSymbol);
        explained = await exports.api.explain(`{ links(where: { _and: [{ id: { _eq: ${exports.itemReplaceSymbol} } }, ${gql}] }, limit: 1) { id } }`);
        sql = explained?.data?.[0]?.sql;
        if (sql) {
            const convertedSql = `SELECT json_array_length("root") as "root" FROM (${sql}) as "root"`;
            const boolExp = await deep.select({ link_id: { _eq: boolExpId } }, { table: 'bool_exp', returning: 'id link_id' });
            const boolExpLink = boolExp?.data?.[0];
            log('boolExpLink', boolExpLink);
            if (boolExpLink) {
                const mutateResult = await deep.update(boolExpLink.id, {
                    value: convertedSql,
                }, { table: 'bool_exp' });
            }
            else {
                const mutateResult = await deep.insert({
                    link_id: boolExpId,
                    value: convertedSql,
                }, { table: 'bool_exp' });
            }
        }
    }
    catch (e) {
        error(e);
        error('error', gql, explained, sql);
    }
};
exports.boolExpToSQL = boolExpToSQL;
//# sourceMappingURL=bool_exp_to_sql.js.map