"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSerialOperation = exports.generateSerial = void 0;
const debug_1 = __importDefault(require("debug"));
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const debug = (0, debug_1.default)('deeplinks:gql:serial');
const log = debug.extend('log');
const error = debug.extend('error');
;
;
const generateSerial = ({ actions = [], name = 'SERIAL', alias = 'm', ...options }) => {
    log('generateSerial', { name, alias, actions });
    const calledActions = actions.map((m, i) => typeof (m) === 'function' ? m(alias, i) : m);
    const defs = calledActions.map(m => m.defs.join(',')).join(',');
    const mutationString = `mutation ${name} (${defs}) { ${calledActions.map(m => `${m.resultAlias}: ${m.queryName}(${m.args.join(',')}) { ${m.resultReturning} }`).join('')} }`;
    const mutation = (0, graphql_tag_1.default) `${mutationString}`;
    const variables = {};
    for (let a = 0; a < calledActions.length; a++) {
        const action = calledActions[a];
        for (const v in action.resultVariables) {
            if (Object.prototype.hasOwnProperty.call(action.resultVariables, v)) {
                const variable = action.resultVariables[v];
                variables[v] = variable;
            }
        }
    }
    const result = {
        mutation,
        variables,
        mutationString,
        ...options
    };
    log('generateSerialResult', JSON.stringify({ mutation: mutationString, variables }, null, 2));
    return result;
};
exports.generateSerial = generateSerial;
function createSerialOperation(params) {
    return params;
}
exports.createSerialOperation = createSerialOperation;
//# sourceMappingURL=serial.js.map