"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteIds = exports.deleteId = exports.deleteSelector = exports.deleteHandler = exports.insertSelectorItem = exports.insertSelectorItems = exports.insertSelector = exports.insertHandler = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const client_js_2 = require("./client.js");
const lodash_1 = __importDefault(require("lodash"));
const apolloClient = (0, client_js_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({ apolloClient });
const insertHandler = async (handleOperationTypeId, typeId, code, forceOwnerId, supportsId) => {
    const syncTextFileTypeId = await deep.id('@deep-foundation/core', 'SyncTextFile');
    const handlerJSFile = (await deep.insert({
        type_id: syncTextFileTypeId,
    }, { name: 'INSERT_HANDLER_JS_FILE' })).data[0];
    const handlerJSFileValue = (await deep.insert({ link_id: handlerJSFile?.id, value: code }, { table: 'strings' })).data[0];
    const handlerTypeId = await deep.id('@deep-foundation/core', 'Handler');
    const handler = (await deep.insert({
        from_id: supportsId || await deep.id('@deep-foundation/core', 'dockerSupportsJs'),
        type_id: handlerTypeId,
        to_id: handlerJSFile?.id,
    }, { name: 'INSERT_HANDLER' })).data[0];
    const containTypeId = await deep.id('@deep-foundation/core', 'Contain');
    const ownerId = forceOwnerId || (await deep.id('deep', 'admin'));
    const ownerContainHandler = (await deep.insert({
        from_id: ownerId,
        type_id: containTypeId,
        to_id: handler?.id,
    }, { name: 'INSERT_ADMIN_CONTAIN_HANDLER' })).data[0];
    const handleOperation = (await deep.insert({
        from_id: typeId,
        type_id: handleOperationTypeId,
        to_id: handler?.id,
    }, { name: 'INSERT_INSERT_HANDLER' })).data[0];
    return {
        handlerId: handler?.id,
        handleOperationId: handleOperation?.id,
        handlerJSFileId: handlerJSFile?.id,
        handlerJSFileValueId: handlerJSFileValue?.id,
        ownerContainHandlerId: ownerContainHandler?.id,
    };
};
exports.insertHandler = insertHandler;
async function insertSelector() {
    const typeTypeId = await deep.id('@deep-foundation/core', 'Type');
    const { data: [{ id: ty0 }] } = await deep.insert({
        type_id: typeTypeId,
    });
    const { data: [{ id: ty1 }] } = await deep.insert({
        type_id: typeTypeId,
        from_id: ty0,
        to_id: ty0,
    });
    const treeIncludeDownTypeId = await deep.id('@deep-foundation/core', 'TreeIncludeDown');
    const { data: [{ id: tr1, out: treeIncludes }] } = await deep.insert({
        type_id: await deep.id('@deep-foundation/core', 'Tree'),
        out: { data: [
                {
                    type_id: treeIncludeDownTypeId,
                    to_id: ty1,
                },
                {
                    type_id: treeIncludeDownTypeId,
                    to_id: ty0,
                },
            ] }
    }, {
        returning: `
      id
      out(where: { to_id: { _in: [${ty0}, ${ty1}] }, type_id: { _eq: ${treeIncludeDownTypeId} } }) {
        id
      }
    `,
    });
    const { data: [{ id: id0 }] } = await deep.insert({
        type_id: ty0,
    });
    const selectorIncludeTypeId = await deep.id('@deep-foundation/core', 'SelectorInclude');
    const selectorTreeTypeId = await deep.id('@deep-foundation/core', 'SelectorTree');
    const { data: [{ id: s1, out: selectorData }] } = await deep.insert({
        type_id: await deep.id('@deep-foundation/core', 'Selector'),
        out: { data: [
                {
                    type_id: selectorIncludeTypeId,
                    to_id: id0,
                    out: { data: {
                            type_id: selectorTreeTypeId,
                            to_id: tr1
                        } },
                },
            ] }
    }, {
        returning: `
      id
      out(where: { to_id: { _eq: ${id0} }, type_id: { _eq: ${selectorIncludeTypeId} } }) {
        id
        out(where: { to_id: { _eq: ${tr1} }, type_id: { _eq: ${selectorTreeTypeId} } }) {
          id
        }
      }
    `,
    });
    return {
        nodeTypeId: ty0,
        linkTypeId: ty1,
        treeId: tr1,
        treeIncludesIds: treeIncludes.map(({ id }) => id),
        selectorId: s1,
        selectorIncludeId: selectorData[0].id,
        selectorTreeId: selectorData[0].out[0].id,
        rootId: id0,
    };
}
exports.insertSelector = insertSelector;
;
async function insertSelectorItems({ selectorId, nodeTypeId, linkTypeId, treeId, rootId }) {
    // const { data: [{ id: id1 }] } = await deep.insert({
    //   type_id: nodeTypeId,
    //   in: { data: {
    //     type_id: linkTypeId,
    //     from_id: rootId,
    //   } }
    // });
    const { data: [{ id: linkId1, to: { id: nodeId1 } }] } = await deep.insert({
        type_id: linkTypeId,
        from_id: rootId,
        to: { data: {
                type_id: nodeTypeId,
            } }
    }, { returning: 'id to { id }' });
    // return linkId1; // doesn't work
    // const { data: [{ id: id2 }] } = await deep.insert({
    //   type_id: nodeTypeId,
    //   in: { data: {
    //     type_id: linkTypeId,
    //     from_id: id1,
    //   } }
    // });
    const { data: [{ id: linkId2, to: { id: nodeId2 } }] } = await deep.insert({
        from_id: nodeId1,
        type_id: linkTypeId,
        to: { data: {
                type_id: nodeTypeId,
            } }
    }, { returning: 'id to { id }' });
    // const n1 = await deep.select({
    //   item_id: { _eq: id2 }, selector_id: { _eq: selectorId }
    // }, { table: 'selectors', returning: 'item_id selector_id' });
    // assert.lengthOf(n1?.data, 1, `item_id ${id2} must be in selector_id ${selectorId}`);
    // return linkId2;
    return [
        { linkId: linkId1, nodeId: nodeId1 },
        { linkId: linkId2, nodeId: nodeId2 },
    ];
}
exports.insertSelectorItems = insertSelectorItems;
;
async function insertSelectorItem({ selectorId, nodeTypeId, linkTypeId, treeId, rootId }) {
    const { data: [{ id: linkId }] } = await deep.insert({
        type_id: linkTypeId,
        from_id: rootId,
        to: { data: {
                type_id: nodeTypeId,
            } }
    }, { returning: 'id to { id }' });
    return { linkId };
}
exports.insertSelectorItem = insertSelectorItem;
;
const deleteHandler = async (handler) => {
    const { handlerJSFileValueId, scheduleValueId, ...ids } = handler;
    const result = { links: [], strings: [] };
    const _ids = [];
    const compact = { id: { _in: _ids.concat(lodash_1.default.compact(Object.values(ids))) } };
    result.links.push(await deep.delete(compact));
    return result;
};
exports.deleteHandler = deleteHandler;
const deleteSelector = async (selector) => {
    const { treeIncludesIds, ...withoutTreeIncluds } = selector;
    const ids = (lodash_1.default.concat(treeIncludesIds, Object.values(withoutTreeIncluds)));
    const _ids = [];
    const compact = { id: { _in: _ids.concat(lodash_1.default.compact(ids)) } };
    await deep.delete(compact);
};
exports.deleteSelector = deleteSelector;
async function deleteId(id, options = { table: 'links' }) {
    await deleteIds([id], options);
}
exports.deleteId = deleteId;
async function deleteIds(ids, options = { table: 'links' }) {
    // return await deep.delete(ids, options); // should work, but doesn't
    const idsFiltered = ids?.filter(linkId => typeof linkId === 'number');
    if (idsFiltered?.length > 0) {
        // log(`${options.table}, deleteIds[0..${idsFiltered.length}]: ${idsFiltered.join(', ')}`);
        try {
            return await deep.delete(idsFiltered, options);
        }
        catch (e) {
            console.error(`Error deleting ids: ${idsFiltered.join(', ')}`, JSON.stringify(e, null, 2));
        }
    }
    else {
        return { data: [] };
    }
}
exports.deleteIds = deleteIds;
//# sourceMappingURL=handlers.js.map