"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.minilinksQueryComparison = exports.minilinksQueryLevel = exports.minilinksQueryHandle = exports.minilinksQueryIs = exports.minilinksQuery = void 0;
const client_js_1 = require("./client.js");
const minilinksQuery = (query, ml) => {
    if (typeof (query) === 'number')
        return [ml.byId[query]];
    else {
        const q = (0, client_js_1.serializeQuery)(query);
        const result = (0, exports.minilinksQueryHandle)(q.where, ml);
        return q.limit ? result.slice(q.offset || 0, (q.offset || 0) + (q.limit)) : result;
    }
};
exports.minilinksQuery = minilinksQuery;
const minilinksQueryIs = (query, link) => {
    if (typeof (query) === 'number')
        return link.id === query;
    else {
        const q = (0, client_js_1.serializeQuery)(query);
        return (0, exports.minilinksQueryLevel)(q, link, 'links');
    }
};
exports.minilinksQueryIs = minilinksQueryIs;
const minilinksQueryHandle = (q, ml) => {
    return ml.links.filter((link) => {
        const r = (0, exports.minilinksQueryLevel)(q, link, 'links');
        return r;
    });
};
exports.minilinksQueryHandle = minilinksQueryHandle;
const minilinksQueryLevel = (q, link, env = 'links') => {
    const fields = Object.keys(q);
    if (env === 'links') {
        for (const f in fields) {
            const field = fields[f];
            if (field === '_and') {
                for (const a in q[field]) {
                    if (!(0, exports.minilinksQueryLevel)(q[field][a], link, env)) {
                        return false;
                    }
                }
            }
            else if (field === '_or') {
                let oneOf = false;
                for (const a in q[field]) {
                    if ((0, exports.minilinksQueryLevel)(q[field][a], link, env)) {
                        oneOf = true;
                        break;
                    }
                }
                return oneOf;
            }
            else if (field === '_not') {
                if ((0, exports.minilinksQueryLevel)(q[field], link, env)) {
                    return false;
                }
            }
            else if (client_js_1._serialize?.[env]?.fields?.[field]) {
                if (!(0, exports.minilinksQueryComparison)(q, link, field, env)) {
                    return false;
                }
            }
            else {
                if (client_js_1._serialize?.[env]?.relations?.[field] === 'links') {
                    if (typeof (q[field]) === 'object') {
                        if (typeof (q[field].length) === 'number') {
                            if (!q[field].length)
                                return false;
                            // at least one subfield must be true
                            let found = false;
                            for (const sub in link?.[field]) {
                                const sublink = link?.[field][sub];
                                if ((0, exports.minilinksQueryLevel)(q[field], sublink, env)) {
                                    found = true;
                                    break;
                                }
                            }
                            if (!found)
                                return false;
                        }
                        else {
                            if (Array.isArray(link?.[field])) {
                                // at least one subfield must be true
                                let found = false;
                                for (const el in link?.[field]) {
                                    if ((0, exports.minilinksQueryLevel)(q[field], link?.[field][el], env)) {
                                        found = true;
                                        break;
                                    }
                                }
                                if (!found)
                                    return false;
                            }
                            else {
                                if (!(0, exports.minilinksQueryLevel)(q[field], link?.[field], env)) {
                                    return false;
                                }
                            }
                        }
                    }
                }
                else if (client_js_1._serialize?.[env]?.relations?.[field] === 'value') {
                    if (!link?.[field] || !(0, exports.minilinksQueryLevel)(q[field], link?.[field], 'value')) {
                        return false;
                    }
                }
                else {
                    return false;
                }
            }
        }
    }
    else if (env === 'value') {
        for (const f in fields) {
            const field = fields[f];
            if (client_js_1._serialize?.[env]?.fields?.[field]) {
                if (!(0, exports.minilinksQueryComparison)(q, link, field, env)) {
                    return false;
                }
            }
            else {
                return false;
            }
        }
    }
    else {
        return false;
    }
    return true;
};
exports.minilinksQueryLevel = minilinksQueryLevel;
const minilinksQueryComparison = (q, link, field, env = 'links') => {
    const comp = q?.[field];
    if (typeof (comp) === 'undefined')
        throw new Error(`${field} === undefined`);
    if (typeof (link?.[field]) === 'object' && Array.isArray(link?.[field])) {
        if (comp.hasOwnProperty('_eq')) {
            if (!link?.[field].includes(comp._eq))
                return false;
        }
        if (comp.hasOwnProperty('_neq')) {
            if (link?.[field].includes(comp._neq))
                return false;
        }
    }
    else {
        if (typeof (link?.[field]) === 'undefined')
            return false;
        if (comp.hasOwnProperty('_eq')) {
            if (link?.[field] !== comp._eq)
                return false;
        }
        if (comp.hasOwnProperty('_neq')) {
            if (link?.[field] === comp._neq)
                return false;
        }
        if (comp.hasOwnProperty('_gt')) {
            if (!(link?.[field] > comp._gt))
                return false;
        }
        if (comp.hasOwnProperty('_gte')) {
            if (!(link?.[field] >= comp._gte))
                return false;
        }
        if (comp.hasOwnProperty('_lt')) {
            if (!(link?.[field] < comp._lt))
                return false;
        }
        if (comp.hasOwnProperty('_lte')) {
            if (!(link?.[field] <= comp._lte))
                return false;
        }
        if (comp.hasOwnProperty('_is_null')) {
            if ((link?.[field] === null) === comp._is_null)
                return false;
        }
        if (comp.hasOwnProperty('_in')) {
            if (!comp?._in?.includes(link?.[field]))
                return false;
        }
        if (comp.hasOwnProperty('_nin')) {
            if (comp?._nin?.includes(link?.[field]))
                return false;
        }
        if (comp.hasOwnProperty('_ilike')) {
            if (!link?.[field]?.toLowerCase || !link?.[field]?.toLowerCase()?.includes(comp?._ilike?.toLowerCase()))
                return false;
        }
        if (comp.hasOwnProperty('_nilike')) {
            if (!link?.[field]?.toLowerCase || !link?.[field]?.toLowerCase()?.includes(comp?._nilike?.toLowerCase()))
                return false;
        }
        if (comp.hasOwnProperty('_like')) {
            if (!link?.[field]?.includes(comp?._like))
                return false;
        }
        if (comp.hasOwnProperty('_nlike')) {
            if (!link?.[field]?.includes(comp?._nlike))
                return false;
        }
        if (comp.hasOwnProperty('_iregex')) {
            if (!link?.[field]?.toLowerCase || !link?.[field]?.toLowerCase().match(comp?._iregex))
                return false;
        }
        if (comp.hasOwnProperty('_regex')) {
            if (!link?.[field].match(comp?._regex))
                return false;
        }
        if (comp.hasOwnProperty('_niregex')) {
            if (!link?.[field]?.toLowerCase || !link?.[field]?.toLowerCase().match(comp?._niregex))
                return false;
        }
        if (comp.hasOwnProperty('_nregex')) {
            if (!link?.[field].match(comp?._nregex))
                return false;
        }
        return true;
    }
};
exports.minilinksQueryComparison = minilinksQueryComparison;
//# sourceMappingURL=minilinks-query.js.map