"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolve = exports.reject = exports.findPromiseLink = exports.awaitPromise = exports.delay = void 0;
const debug_1 = __importDefault(require("debug"));
const index_js_1 = require("./gql/index.js");
const debug = (0, debug_1.default)('deeplinks:promise');
const log = debug.extend('log');
const error = debug.extend('error');
// Force enable this file errors output
const delay = (time) => new Promise(res => setTimeout(() => res(null), time));
exports.delay = delay;
function awaitPromise(options) {
    const id = options.id;
    const timeout = options.timeout || 1000; // @TODO todo dynamic timeout based on handlers avg runtime
    const client = options.client;
    return new Promise(async (res, rej) => {
        log('promise', { id });
        if (!id) {
            log('!id', { id: options.id });
            return rej('options.id must be defined!');
        }
        let promises = null;
        try {
            while (true) {
                const result = await client.query((0, index_js_1.generateQuery)({
                    queries: [(0, index_js_1.generateQueryData)({ tableName: 'links', returning: `
          id type { id value } from_id to_id
          `, variables: {
                                where: {
                                    _or: [
                                        // { id: { _eq: id } },
                                        {
                                            from_id: { _eq: id },
                                            type_id: { _eq: options.Then },
                                            to: {
                                                _not: {
                                                    out: {
                                                        type_id: { _in: [options.Resolved, options.Rejected] }
                                                    }
                                                }
                                            }
                                        },
                                        {
                                            type_id: { _eq: options.Promise },
                                            in: { type_id: { _eq: options.Then }, from_id: { _eq: id } },
                                        },
                                        {
                                            from: {
                                                type_id: { _eq: options.Promise },
                                                in: { type_id: { _eq: options.Then }, from_id: { _eq: id } },
                                            },
                                            type_id: { _in: [options.Resolved, options.Rejected] },
                                        },
                                    ],
                                },
                            } })],
                    name: 'PROMISE',
                }));
                log('result', JSON.stringify(result, null, 2));
                try {
                    if (result?.errors) {
                        log('error', result?.errors);
                        return rej(result?.errors);
                    }
                    else if (result?.data) {
                        const links = result.data?.q0;
                        log('data', JSON.stringify(links, null, 2));
                        if (promises === null) {
                            promises = {};
                            for (let l = 0; l < links.length; l++) {
                                const link = links[l];
                                if (link?.type?.id === options.Then)
                                    promises[link?.to_id] = true;
                            }
                        }
                        let promiseResolves = {};
                        let promiseRejects = {};
                        for (let l = 0; l < links.length; l++) {
                            const link = links[l];
                            if (link?.type?.id === options.Resolved)
                                promiseResolves[link?.from_id] = true;
                            else if (link?.type?.id === options.Rejected)
                                promiseRejects[link?.from_id] = true;
                        }
                        let promisesCount = Object.keys(promises).length;
                        let thenExists = promisesCount > 0;
                        log('analized', { thenExists });
                        if (thenExists) {
                            let thenResolvedByPromise = [];
                            let thenRejectedByPromise = [];
                            for (let key in promises) {
                                thenResolvedByPromise.push(!!promiseResolves[key]);
                                thenRejectedByPromise.push(!!promiseRejects[key]);
                            }
                            const thenResolved = thenResolvedByPromise.some(r => r);
                            const thenRejected = thenRejectedByPromise.some(r => r);
                            log('analized', { thenResolved, thenRejected });
                            const filteredLinks = links.filter(l => l?.id === id ||
                                l?.type?.id === options.Then ||
                                (l?.type?.id === options.Promise && promises[l?.id]) ||
                                (l?.type?.id === options.Resolved && promises[l?.from_id]) ||
                                (l?.type?.id === options.Rejected && promises[l?.from_id]));
                            log('filteredLinks', JSON.stringify(filteredLinks, null, 2));
                            if (thenResolved && !thenRejected) {
                                log('resolved');
                                return res(options.Results ? filteredLinks : true);
                            }
                            else if (thenRejected) {
                                log('rejected');
                                return res(options.Results ? filteredLinks : false);
                            }
                            else {
                                log('waiting');
                            }
                        }
                        else {
                            log('!then');
                            return res(options.Results ? [] : true);
                        }
                    }
                }
                catch (e) {
                    error('error', e);
                    return rej(options.Results ? e : false);
                }
                await (0, exports.delay)(timeout);
            }
        }
        catch (e) {
            error('error', e);
            return rej(options.Results ? e : false);
        }
    });
}
exports.awaitPromise = awaitPromise;
;
async function findPromiseLink(options) {
    const result = await options.client.query((0, index_js_1.generateQuery)({
        queries: [
            (0, index_js_1.generateQueryData)({ tableName: 'links', returning: `id`, variables: { where: {
                        type_id: { _eq: options.Promise },
                        in: {
                            type_id: { _eq: options.Then },
                            from_id: { _eq: options.id },
                            to: {
                                _not: {
                                    out: {
                                        type_id: { _in: [options.Resolved, options.Rejected] }
                                    }
                                }
                            }
                        },
                    } } }),
        ],
        name: 'FIND_PROMISE',
    }));
    return result?.data?.q0?.[0];
}
exports.findPromiseLink = findPromiseLink;
async function reject(options) {
    log('reject', options.id);
    const promise = await findPromiseLink(options);
    if (promise) {
        log('rejected', await options.client.mutate((0, index_js_1.generateSerial)({
            actions: [(0, index_js_1.insertMutation)('links', { objects: { type_id: options.Rejected, from_id: promise.id, to_id: promise.id } })],
            name: 'REJECT',
        })));
        return true;
    }
    return false;
}
exports.reject = reject;
async function resolve(options) {
    log('resolve', options.id);
    const promise = await findPromiseLink(options);
    if (promise) {
        log('resolved', await options.client.mutate((0, index_js_1.generateSerial)({
            actions: [(0, index_js_1.insertMutation)('links', { objects: { type_id: options.Resolved, from_id: promise.id, to_id: promise.id } })],
            name: 'RESOLVE',
        })));
        return true;
    }
    return false;
}
exports.resolve = resolve;
//# sourceMappingURL=promise.js.map