"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.typeDefs = exports.typeDefsString = void 0;
const jwt_js_1 = require("../jwt.js");
const client_js_1 = require("@deep-foundation/hasura/client.js");
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const apollo_server_express_1 = require("apollo-server-express");
const client_js_2 = require("../client.js");
const apollo_server_core_1 = require("apollo-server-core");
const JWT_SECRET = process.env.JWT_SECRET;
const jwt_secret = JSON.parse(JWT_SECRET);
exports.typeDefsString = `
  type Query {
    guest: GuestOutput
  }
  type GuestOutput {
    token: String
    linkId: Int
  }
`;
exports.typeDefs = (0, graphql_tag_1.default) `${exports.typeDefsString}`;
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
}, { ApolloClient: { defaultOptions: { query: { fetchPolicy: 'no-cache' }, watchQuery: { fetchPolicy: 'no-cache' } } } });
const deep = new client_js_2.DeepClient({
    apolloClient: client,
});
const resolvers = {
    Query: {
        guest: async (source, args, context, info) => {
            const { data: [{ id }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'User'),
                out: { data: [
                        {
                            type_id: await deep.id('@deep-foundation/core', 'Join'),
                            to_id: await deep.id('deep', 'users')
                        },
                    ] },
            });
            await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Rule'),
                out: { data: [
                        {
                            type_id: await deep.id('@deep-foundation/core', 'RuleSubject'),
                            to: { data: {
                                    type_id: await deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                                            to_id: id,
                                            out: { data: {
                                                    type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                                                    to_id: await deep.id('@deep-foundation/core', 'containTree'),
                                                } },
                                        } }
                                } }
                        },
                        {
                            type_id: await deep.id('@deep-foundation/core', 'RuleObject'),
                            to: { data: {
                                    type_id: await deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                                            to_id: id,
                                            out: { data: {
                                                    type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                                                    to_id: await deep.id('@deep-foundation/core', 'containTree'),
                                                } },
                                        } }
                                } }
                        },
                        {
                            type_id: await deep.id('@deep-foundation/core', 'RuleAction'),
                            to: { data: {
                                    type_id: await deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                                            to_id: await deep.id('@deep-foundation/core', 'AllowSelect'),
                                            out: { data: {
                                                    type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                                                    to_id: await deep.id('@deep-foundation/core', 'containTree'),
                                                } },
                                        } }
                                } }
                        },
                    ] },
            });
            const token = (0, jwt_js_1.jwt)({
                secret: jwt_secret.key,
                linkId: id,
            });
            return { token, linkId: id };
        },
    }
};
const context = ({ req }) => {
    return { headers: req.headers };
};
const generateApolloServer = (httpServer) => {
    return new apollo_server_express_1.ApolloServer({
        introspection: true,
        typeDefs: exports.typeDefs,
        resolvers,
        context,
        plugins: [
            (0, apollo_server_core_1.ApolloServerPluginDrainHttpServer)({ httpServer }),
            (0, apollo_server_core_1.ApolloServerPluginLandingPageGraphQLPlayground)()
        ]
    });
};
exports.default = generateApolloServer;
//# sourceMappingURL=guest.js.map