"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.typeDefs = exports.typeDefsString = void 0;
const jwt_js_1 = require("../jwt.js");
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const apollo_server_express_1 = require("apollo-server-express");
const apollo_server_core_1 = require("apollo-server-core");
const client_js_1 = require("@deep-foundation/hasura/client.js");
const client_js_2 = require("../client.js");
const apolloClient = (0, client_js_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({ apolloClient });
const JWT_SECRET = process.env.JWT_SECRET;
const jwt_secret = JSON.parse(JWT_SECRET);
exports.typeDefsString = `
  type Query {
    jwt(input: JWTInput): JWTOutput
  }
  input JWTInput {
    linkId: Int
  }
  type JWTOutput {
    token: String
    linkId: Int
    error: String
  }
`;
exports.typeDefs = (0, graphql_tag_1.default) `${exports.typeDefsString}`;
const resolvers = {
    Query: {
        jwt: async (source, args, context, info) => {
            const { linkId } = args.input;
            try {
                if (!linkId) {
                    return {
                        linkId: +context?.headers?.['x-hasura-user-id']
                    };
                }
                if (!+context?.headers?.['x-hasura-user-id'] && context?.headers?.['x-hasura-role'] !== 'admin') {
                    return { error: '!currentUser' };
                }
                if (context?.headers?.['x-hasura-role'] !== 'admin' &&
                    !(await deep.select({
                        subject_id: { _eq: +context?.headers?.['x-hasura-user-id'] },
                        action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowAdmin') },
                    }, { table: 'can', returning: 'rule_id' }))?.data?.[0] &&
                    +context?.headers?.['x-hasura-user-id'] !== linkId &&
                    !await deep.can(linkId, +context?.headers?.['x-hasura-user-id'], await deep.id('@deep-foundation/core', 'AllowLogin'))) {
                    return { error: 'cant' };
                }
                const token = (0, jwt_js_1.jwt)({
                    secret: jwt_secret.key,
                    linkId,
                    role: await deep.can(null, linkId, await deep.id('@deep-foundation/core', 'AllowAdmin')) ? 'admin' : 'link',
                });
                return { token, linkId };
            }
            catch (error) {
                return { error };
            }
        },
    }
};
const context = ({ req }) => {
    return { headers: req.headers };
};
const generateApolloServer = (httpServer) => {
    return new apollo_server_express_1.ApolloServer({
        introspection: true,
        typeDefs: exports.typeDefs,
        resolvers,
        context,
        plugins: [
            (0, apollo_server_core_1.ApolloServerPluginDrainHttpServer)({ httpServer }),
            (0, apollo_server_core_1.ApolloServerPluginLandingPageGraphQLPlayground)()
        ]
    });
};
exports.default = generateApolloServer;
//# sourceMappingURL=jwt.js.map