"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const api_js_1 = require("@deep-foundation/hasura/api.js");
const index_js_1 = require("../gql/index.js");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:eh:reserved');
const log = debug.extend('log');
const error = debug.extend('error');
// Force enable this file errors output
const SCHEMA = 'public';
exports.api = new api_js_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
exports.default = async (req, res) => {
    try {
        const count = req?.body?.input?.count;
        if (!count)
            return res.status(500).json({ error: 'no count' });
        // const token = req?.body?.session_variables;
        const links = [];
        for (let i = 0; i < count; i++)
            links[i] = { type_id: 0 };
        const mutateLinksResult = await client.mutate((0, index_js_1.generateSerial)({
            actions: [(0, index_js_1.insertMutation)('links', { objects: links })],
            name: 'INSERT_LINKS',
        }));
        const ids = mutateLinksResult.data['m0']?.returning?.map(node => node.id);
        if (!ids)
            res.status(500).json({ error: 'insert links error' });
        const mutateReservedResult = await client.mutate((0, index_js_1.generateSerial)({
            actions: [
                (0, index_js_1.generateMutation)({
                    tableName: 'reserved', operation: 'insert',
                    variables: { objects: { reserved_ids: ids, user_id: 123123, created_at: new Date().toISOString() } }, //userid
                }),
            ],
            name: 'INSERT_RESERVED',
        }));
        if (!mutateLinksResult.data['m0']?.returning?.[0]?.id)
            res.status(500).json({ error: 'insert reserved error' });
        return res.json({ ids });
    }
    catch (e) {
        error(JSON.stringify(e, null, 2));
        return res.status(500).json({ error: e.toString() });
    }
};
//# sourceMappingURL=reserved.js.map