"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.TABLE_NAME = exports.SCHEMA = exports.api = void 0;
const api_js_1 = require("@deep-foundation/hasura/api.js");
const sql_js_1 = require("@deep-foundation/hasura/sql.js");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:migrations:links');
const log = debug.extend('log');
const error = debug.extend('error');
exports.api = new api_js_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
exports.SCHEMA = 'public';
exports.TABLE_NAME = 'links';
const up = async () => {
    log('up');
    await exports.api.sql((0, sql_js_1.sql) `
    CREATE TABLE ${exports.SCHEMA}."${exports.TABLE_NAME}" (
      id bigint PRIMARY KEY,
      from_id bigint DEFAULT 0,
      to_id bigint DEFAULT 0,
      type_id bigint NOT NULL
    );
    CREATE SEQUENCE ${exports.TABLE_NAME}_id_seq
    AS bigint START WITH 1 INCREMENT BY 1 NO MINVALUE NO MAXVALUE CACHE 1;
    ALTER SEQUENCE ${exports.TABLE_NAME}_id_seq OWNED BY ${exports.SCHEMA}."${exports.TABLE_NAME}".id;
    ALTER TABLE ONLY ${exports.SCHEMA}."${exports.TABLE_NAME}" ALTER COLUMN id SET DEFAULT nextval('${exports.TABLE_NAME}_id_seq'::regclass);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__id_hash ON ${exports.TABLE_NAME} USING hash (id);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__from_id_hash ON ${exports.TABLE_NAME} USING hash (from_id);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__from_id_btree ON ${exports.TABLE_NAME} USING btree (from_id);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__to_id_hash ON ${exports.TABLE_NAME} USING hash (to_id);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__to_id_btree ON ${exports.TABLE_NAME} USING btree (to_id);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__type_id_hash ON ${exports.TABLE_NAME} USING hash (type_id);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__type_id_btree ON ${exports.TABLE_NAME} USING btree (type_id);
  `);
    await exports.api.sql((0, sql_js_1.sql) `
    CREATE TABLE ${exports.SCHEMA}."${exports.TABLE_NAME}__tables" (id bigint PRIMARY KEY, name TEXT);
    CREATE SEQUENCE ${exports.TABLE_NAME}__tables_id_seq
    AS bigint START WITH 1 INCREMENT BY 1 NO MINVALUE NO MAXVALUE CACHE 1;
    ALTER SEQUENCE ${exports.TABLE_NAME}__tables_id_seq OWNED BY ${exports.SCHEMA}."${exports.TABLE_NAME}__tables".id;
    ALTER TABLE ONLY ${exports.SCHEMA}."${exports.TABLE_NAME}__tables" ALTER COLUMN id SET DEFAULT nextval('${exports.TABLE_NAME}__tables_id_seq'::regclass);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__tables__id_hash ON ${exports.TABLE_NAME}__tables USING hash (id);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__tables__name_hash ON ${exports.TABLE_NAME}__tables USING hash (name);
    CREATE INDEX IF NOT EXISTS ${exports.TABLE_NAME}__tables__name_btree ON ${exports.TABLE_NAME}__tables USING btree (name);
  `);
    await exports.api.query({
        type: 'track_table',
        args: {
            schema: exports.SCHEMA,
            name: `${exports.TABLE_NAME}__tables`,
        },
    });
    await exports.api.sql((0, sql_js_1.sql) `CREATE OR REPLACE FUNCTION ${exports.TABLE_NAME}__value__function(link ${exports.TABLE_NAME}) RETURNS jsonb STABLE AS $function$ DECLARE
    exists int; str json; num json; obj json;
  BEGIN
    SELECT json_agg(t) FROM "strings" as t INTO str WHERE t."link_id"=link."id";
    SELECT json_agg(t) FROM "numbers" as t INTO num WHERE t."link_id"=link."id";
    SELECT json_agg(t) FROM "objects" as t INTO obj WHERE t."link_id"=link."id";
    IF (str IS NOT NULL) THEN RETURN (str->0); END IF;
    IF (num IS NOT NULL) THEN RETURN (num->0); END IF;
    IF (obj IS NOT NULL) THEN RETURN (obj->0); END IF;
    RETURN NULL;
  END; $function$ LANGUAGE plpgsql;`);
    await exports.api.query({
        type: 'track_table',
        args: {
            schema: exports.SCHEMA,
            name: exports.TABLE_NAME,
        },
    });
    await exports.api.query({
        type: 'add_computed_field',
        args: {
            table: exports.TABLE_NAME,
            source: 'default',
            name: 'value',
            definition: {
                function: {
                    name: `${exports.TABLE_NAME}__value__function`,
                    schema: 'public',
                },
                table_argument: 'link',
            }
        },
    });
    await exports.api.query({
        type: 'create_event_trigger',
        args: {
            name: exports.TABLE_NAME,
            table: exports.TABLE_NAME,
            webhook: `${process.env.MIGRATIONS_DEEPLINKS_URL}/api/links`,
            insert: {
                columns: "*",
                payload: '*',
            },
            update: {
                columns: '*',
                payload: '*',
            },
            delete: {
                columns: '*'
            },
            replace: false,
        },
    });
    await exports.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'from',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: exports.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        from_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await exports.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'to',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: exports.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        to_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await exports.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'type',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: exports.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        type_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await exports.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'in',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: exports.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'to_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    await exports.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'out',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: exports.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'from_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    await exports.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'typed',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: exports.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'type_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
};
exports.up = up;
const down = async () => {
    log('down');
    await exports.api.query({
        type: 'drop_computed_field',
        args: {
            table: exports.TABLE_NAME,
            source: 'default',
            name: 'value',
            cascade: true,
        },
    });
    await exports.api.sql((0, sql_js_1.sql) `DROP FUNCTION IF EXISTS ${exports.TABLE_NAME}__value__function CASCADE;`);
    await exports.api.query({
        type: 'untrack_table',
        args: {
            table: {
                schema: exports.SCHEMA,
                name: exports.TABLE_NAME,
            },
            cascade: true,
        },
    });
    await exports.api.sql((0, sql_js_1.sql) `
    DROP TABLE ${exports.SCHEMA}."${exports.TABLE_NAME}" CASCADE;
  `);
    const tables = ((await exports.api.sql((0, sql_js_1.sql) `
    SELECT * FROM ${exports.TABLE_NAME}__tables;
  `))?.data?.result || [])?.slice(1);
    await exports.api.query({
        type: 'untrack_table',
        args: {
            table: {
                schema: exports.SCHEMA,
                name: `${exports.TABLE_NAME}__tables`,
            },
            cascade: true,
        },
    });
    if (!!tables.length)
        await exports.api.sql((0, sql_js_1.sql) `
    ${tables.map(table => `DROP TABLE IF EXISTS ${table[1]} CASCADE;`).join('')}
  `);
    await exports.api.sql((0, sql_js_1.sql) `
    DROP TABLE ${exports.TABLE_NAME}__tables CASCADE;
  `);
};
exports.down = down;
//# sourceMappingURL=1616701513782-links.js.map