"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = void 0;
const api_js_1 = require("@deep-foundation/hasura/api.js");
const client_js_1 = require("@deep-foundation/hasura/client.js");
const sql_js_1 = require("@deep-foundation/hasura/sql.js");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:migrations:auto-indexes');
const log = debug.extend('log');
const error = debug.extend('error');
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const api = new api_js_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const up = async () => {
    log('up');
    await api.sql((0, sql_js_1.sql) `CREATE OR REPLACE FUNCTION create_btree_index(schema_name text, table_name text, column_name text) RETURNS void
  AS $$
  BEGIN
      EXECUTE 'CREATE INDEX IF NOT EXISTS ' || table_name || '__' || column_name || '_btree ON ' || schema_name || '.' || table_name || ' USING btree (' || column_name || ');';
  END;
  $$ LANGUAGE plpgsql;`);
    await api.sql((0, sql_js_1.sql) `CREATE OR REPLACE FUNCTION create_btree_indexes_for_all_columns(schema__name text, table__name text) RETURNS void
  AS $$
  BEGIN
      PERFORM create_btree_index(table_schema, table_name, column_name) FROM information_schema.columns
      WHERE 
          table_schema = schema__name 
      AND table_name = table__name
      AND data_type != 'text' -- It is not recommended to index 'text' columns with 'btree' index (this index has a limit of key size - 2712 bytes, and 'text' columns can contain larger text strings)
      ;
  END;
  $$ LANGUAGE plpgsql;`);
};
exports.up = up;
const down = async () => {
    log('down');
    await api.sql((0, sql_js_1.sql) `DROP FUNCTION create_btree_index;`);
    await api.sql((0, sql_js_1.sql) `DROP FUNCTION create_btree_indexes_for_all_columns;`);
};
exports.down = down;
//# sourceMappingURL=1621815803560-auto-indexes.js.map