"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.downTable = exports.upTable = exports.RL_TABLE_NAME = void 0;
const debug_1 = __importDefault(require("debug"));
const _1616701513782_links_js_1 = require("./1616701513782-links.js");
const sql_js_1 = require("@deep-foundation/hasura/sql.js");
const debug = (0, debug_1.default)('deeplinks:migrations:reserved-links');
const log = debug.extend('log');
const error = debug.extend('error');
const DEFAULT_SCHEMA = process.env.MIGRATIONS_SCHEMA || 'public';
const DEFAULT_RL_TABLE = process.env.MIGRATIONS_RL_TABLE || 'rl_example__links__reserved';
const DEFAULT_DATE_TYPE_SQL = process.env.MIGRATIONS_DATE_TYPE_SQL || 'timestamp';
const DEFAULT_RL_CRON_SCHEDULE = process.env.DEFAULT_RL_CRON_SCHEDULE || '0 * * * *';
const MIGRATIONS_DEEPLINKS_URL = process.env.MIGRATIONS_DEEPLINKS_URL || 'http://localhost:3006';
exports.RL_TABLE_NAME = 'reserved';
const upTable = async ({ SCHEMA = DEFAULT_SCHEMA, RL_TABLE = DEFAULT_RL_TABLE, DATE_TYPE = DEFAULT_DATE_TYPE_SQL, customColumns = '' } = {}) => {
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    CREATE TABLE ${SCHEMA}."${RL_TABLE}" (id bigint PRIMARY KEY, created_at ${DEFAULT_DATE_TYPE_SQL}, reserved_ids jsonb, user_id bigint${customColumns});
    CREATE SEQUENCE ${RL_TABLE}_id_seq
    AS bigint START WITH 1 INCREMENT BY 1 NO MINVALUE NO MAXVALUE CACHE 1;
    ALTER SEQUENCE ${RL_TABLE}_id_seq OWNED BY ${SCHEMA}."${RL_TABLE}".id;
    ALTER TABLE ONLY ${SCHEMA}."${RL_TABLE}" ALTER COLUMN id SET DEFAULT nextval('${RL_TABLE}_id_seq'::regclass);
  `);
    await _1616701513782_links_js_1.api.query({
        type: 'track_table',
        args: {
            schema: SCHEMA,
            name: RL_TABLE,
        },
    });
};
exports.upTable = upTable;
const downTable = async ({ SCHEMA = DEFAULT_SCHEMA, RL_TABLE = DEFAULT_RL_TABLE } = {}) => {
    await _1616701513782_links_js_1.api.query({
        type: 'untrack_table',
        args: {
            table: {
                schema: SCHEMA,
                name: RL_TABLE,
            },
        },
    });
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP TABLE ${SCHEMA}."${RL_TABLE}";
  `);
};
exports.downTable = downTable;
const up = async () => {
    log('up');
    log('table');
    await (0, exports.upTable)({
        RL_TABLE: exports.RL_TABLE_NAME,
    });
    log('trigger');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `CREATE OR REPLACE FUNCTION ${_1616701513782_links_js_1.TABLE_NAME}__reserved__instead_of_insert__function() RETURNS TRIGGER AS $trigger$
    BEGIN
      IF NEW.id IS NOT NULL THEN
        IF EXISTS( SELECT FROM ${exports.RL_TABLE_NAME} as RL, ${_1616701513782_links_js_1.TABLE_NAME} AS LINKS WHERE RL.reserved_ids @> NEW.id::text::jsonb AND LINKS.type_id = 0 AND LINKS.id = NEW.id) THEN
          DELETE FROM ${_1616701513782_links_js_1.TABLE_NAME} WHERE id = NEW.id;
          RETURN NEW;
        ELSE
          IF (NOT EXISTS (SELECT * FROM "links_id_seq" WHERE last_value >= NEW.id)) THEN
            RAISE EXCEPTION 'Illegal insert link with custom id with { id: %, type_id: %, from_id: %, to_id: % } %.', NEW.id, NEW.type_id, NEW.from_id, NEW.to_id, (SELECT row_to_json(t) FROM "links_id_seq" as t) USING HINT = 'Use reserve action before inserting link with id';
          END IF;
        END IF;
      END IF;
      RETURN NEW;
    END; $trigger$ LANGUAGE plpgsql;`);
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `CREATE TRIGGER ${_1616701513782_links_js_1.TABLE_NAME}__reserved__instead_of_insert__trigger BEFORE INSERT ON "${_1616701513782_links_js_1.TABLE_NAME}" FOR EACH ROW WHEN (pg_trigger_depth() < 1 AND NEW.type_id != 0) EXECUTE PROCEDURE ${_1616701513782_links_js_1.TABLE_NAME}__reserved__instead_of_insert__function();`);
    log('cron_trigger');
    await _1616701513782_links_js_1.api.query({
        type: 'create_cron_trigger',
        args: {
            name: 'reserved_links_cleaner',
            webhook: `${MIGRATIONS_DEEPLINKS_URL}/api/reserved-cleaner`,
            schedule: DEFAULT_RL_CRON_SCHEDULE,
            include_in_metadata: true,
            payload: {},
            retry_conf: {
                num_retries: 3,
                timeout_seconds: 120,
                tolerance_seconds: 21675,
                retry_interval_seconds: 12
            },
            comment: 'clean reserved'
        }
    });
    log('action_types');
    await _1616701513782_links_js_1.api.query({
        type: 'set_custom_types',
        args: {
            scalars: [],
            enums: [],
            input_objects: [],
            objects: [
                {
                    name: 'reserveResponse',
                    fields: [
                        {
                            name: 'ids',
                            type: '[String!]!'
                        }
                    ]
                }
            ]
        }
    });
    log('action');
    await _1616701513782_links_js_1.api.query({
        type: 'create_action',
        args: {
            name: 'reserve',
            definition: {
                kind: 'synchronous',
                type: 'mutation',
                forward_client_headers: true,
                arguments: [
                    {
                        name: 'count',
                        type: 'Int!'
                    },
                ],
                output_type: 'reserveResponse',
                handler: `${MIGRATIONS_DEEPLINKS_URL}/api/reserved`
            }
        }
    });
    await _1616701513782_links_js_1.api.metadata({
        type: 'create_action_permission',
        args: {
            action: 'reserve',
            role: 'link',
        }
    });
};
exports.up = up;
const down = async () => {
    log('down');
    log('action');
    await _1616701513782_links_js_1.api.query({
        type: 'drop_action',
        args: {
            name: 'reserve',
            clear_data: true
        }
    });
    log('cron_trigger');
    await _1616701513782_links_js_1.api.query({
        type: 'delete_cron_trigger',
        args: {
            name: 'reserved_links_cleaner',
        }
    });
    log('trigger');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP TRIGGER IF EXISTS ${_1616701513782_links_js_1.TABLE_NAME}__reserved__instead_of_insert__trigger ON ${_1616701513782_links_js_1.TABLE_NAME} CASCADE;
    DROP FUNCTION IF EXISTS ${_1616701513782_links_js_1.TABLE_NAME}__reserved__instead_of_insert__function() CASCADE;
  `);
    log('table');
    await (0, exports.downTable)({
        RL_TABLE: exports.RL_TABLE_NAME,
    });
};
exports.down = down;
//# sourceMappingURL=1622230000000-reserved-links.js.map