"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.api = exports.BOOL_EXP_TABLE_NAME = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const debug_1 = __importDefault(require("debug"));
const client_js_2 = require("../imports/client.js");
const api_js_1 = require("@deep-foundation/hasura/api.js");
const type_table_js_1 = require("../imports/type-table.js");
exports.BOOL_EXP_TABLE_NAME = 'bool_exp';
const debug = (0, debug_1.default)('deeplinks:migrations:values');
const log = debug.extend('log');
const error = debug.extend('error');
const apolloClient = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
exports.api = new api_js_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({ apolloClient });
const up = async () => {
    log('up');
    await ((0, type_table_js_1.generateUp)({
        schemaName: 'public',
        tableName: 'strings',
        valueType: 'TEXT NOT NULL DEFAULT ""',
        customColumnsSql: 'value text',
        linkRelation: 'string',
        linksTableName: 'links',
        api: exports.api,
        deep,
    })());
    await ((0, type_table_js_1.generateUp)({
        schemaName: 'public',
        tableName: 'numbers',
        valueType: 'numeric NOT NULL DEFAULT 0',
        customColumnsSql: 'value numeric',
        linkRelation: 'number',
        linksTableName: 'links',
        api: exports.api,
        deep,
    })());
    await ((0, type_table_js_1.generateUp)({
        schemaName: 'public',
        tableName: 'objects',
        valueType: 'jsonb NOT NULL DEFAULT \'{}\'::jsonb',
        customColumnsSql: 'value jsonb',
        linkRelation: 'object',
        linksTableName: 'links',
        api: exports.api,
        deep,
    })());
    await ((0, type_table_js_1.generateUp)({
        schemaName: 'public',
        tableName: exports.BOOL_EXP_TABLE_NAME,
        valueType: 'TEXT',
        customColumnsSql: 'value text',
        api: exports.api,
        deep,
    })());
};
exports.up = up;
const down = async () => {
    log('down');
    await ((0, type_table_js_1.generateDown)({
        schemaName: 'public',
        tableName: 'strings',
        valueType: 'TEXT',
        customColumnsSql: 'value text',
        linkRelation: 'string',
        linksTableName: 'links',
        api: exports.api,
        deep,
    })());
    await ((0, type_table_js_1.generateDown)({
        schemaName: 'public',
        tableName: 'numbers',
        valueType: 'float8',
        customColumnsSql: 'value bigint',
        linkRelation: 'number',
        linksTableName: 'links',
        api: exports.api,
        deep,
    })());
    await ((0, type_table_js_1.generateDown)({
        schemaName: 'public',
        tableName: 'objects',
        valueType: 'jsonb',
        customColumnsSql: 'value jsonb',
        linkRelation: 'object',
        linksTableName: 'links',
        api: exports.api,
        deep,
    })());
    await ((0, type_table_js_1.generateDown)({
        schemaName: 'public',
        tableName: exports.BOOL_EXP_TABLE_NAME,
        valueType: 'TEXT',
        customColumnsSql: 'value text',
        api: exports.api,
        deep,
    })());
};
exports.down = down;
//# sourceMappingURL=1622421760250-values.js.map