"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.BOOL_EXP_COMPUTED_FIELD = exports.CACHE = exports.TABLE_NAME = exports.SELECTORS_TABLE_NAME = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const sql_js_1 = require("@deep-foundation/hasura/sql.js");
const debug_1 = __importDefault(require("debug"));
const bool_exp_to_sql_js_1 = require("../imports/bool_exp_to_sql.js");
const client_js_2 = require("../imports/client.js");
const _1616701513782_links_js_1 = require("./1616701513782-links.js");
const _1621815803572_materialized_path_js_1 = require("./1621815803572-materialized-path.js");
const _1622421760250_values_js_1 = require("./1622421760250-values.js");
const debug = (0, debug_1.default)('deeplinks:migrations:selectors');
const log = debug.extend('log');
const error = debug.extend('error');
exports.SELECTORS_TABLE_NAME = 'selectors';
exports.TABLE_NAME = 'links';
exports.CACHE = 'selectors_cache';
exports.BOOL_EXP_COMPUTED_FIELD = `${exports.TABLE_NAME}__exec_bool_exp__function`;
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({
    apolloClient: client,
});
const up = async () => {
    log('up');
    log('view');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    CREATE VIEW ${exports.SELECTORS_TABLE_NAME} AS
    SELECT 
      mp_include."item_id" as "item_id",
      cache_include."selector_id" as "selector_id",
      cache_include."selector_include_id" as "selector_include_id",
      cache_include."selector_filter_bool_exp_id" as "query_id"
    FROM
      ${_1621815803572_materialized_path_js_1.MP_TABLE_NAME} as mp_include,
      ${exports.CACHE} as cache_include
    WHERE
      cache_include."selector_include_id" != 0 AND
      mp_include."path_item_id" = cache_include."link_id" AND
      mp_include."group_id" = cache_include."tree_id" AND
      NOT EXISTS (
        SELECT mp_exclude."id"
        FROM
        ${exports.CACHE} as cache_exclude,
        ${_1621815803572_materialized_path_js_1.MP_TABLE_NAME} as mp_exclude
        WHERE (
          cache_exclude."selector_exclude_id" != 0 AND
          cache_exclude."selector_id" = cache_include."selector_id" AND
          mp_exclude."path_item_id" = cache_exclude."link_id" AND
          mp_exclude."item_id" = mp_include."item_id" AND
          mp_exclude."group_id" = cache_exclude."tree_id"
        )
      );
  `);
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    CREATE OR REPLACE FUNCTION bool_exp_execute(target_link_id bigint, bool_exp_link_id bigint, user_id bigint) RETURNS BOOL AS $trigger$ DECLARE
      boolExp RECORD;
      sqlResult INT;
      query TEXT;
    BEGIN
      SELECT be.* into boolExp
      FROM "${_1622421760250_values_js_1.BOOL_EXP_TABLE_NAME}" as be
      WHERE be.link_id=bool_exp_link_id;
      IF boolExp IS NOT NULL THEN
        IF (user_id IS NULL) THEN
          user_id := ${client_js_2._ids?.['@deep-foundation/core']?.Any};
        END IF;
        SELECT REPLACE(REPLACE(boolExp.value, ${bool_exp_to_sql_js_1.itemReplaceSymbol}::text, target_link_id::text), ${bool_exp_to_sql_js_1.userReplaceSymbol}::text, user_id::text) INTO query;
        EXECUTE query INTO sqlResult;
        IF sqlResult = 0 THEN
          RETURN FALSE;
        END IF;
        RETURN TRUE;
      END IF;
      RETURN NULL;
    END; $trigger$ LANGUAGE plpgsql;
  `);
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `CREATE OR REPLACE FUNCTION ${exports.BOOL_EXP_COMPUTED_FIELD}(link ${exports.TABLE_NAME}, link_id bigint, hasura_session json) RETURNS SETOF links STABLE AS $function$ DECLARE
    RESULT BOOLEAN;
    user_id bigint;
  BEGIN
    user_id := hasura_session::json->>'x-hasura-user-id';
    SELECT INTO RESULT bool_exp_execute(link_id, link.id, user_id);
    IF RESULT THEN
      RETURN QUERY SELECT * FROM links WHERE id=link_id LIMIT 1;
    ELSE
      RETURN;
    END IF;
  END; $function$ LANGUAGE plpgsql;`);
    await _1616701513782_links_js_1.api.query({
        type: 'add_computed_field',
        args: {
            table: exports.TABLE_NAME,
            source: 'default',
            name: 'exec_bool_exp',
            definition: {
                function: {
                    name: exports.BOOL_EXP_COMPUTED_FIELD,
                    schema: 'public',
                },
                table_argument: 'link',
                session_argument: 'hasura_session',
            }
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'track_table',
        args: {
            schema: _1616701513782_links_js_1.SCHEMA,
            name: exports.SELECTORS_TABLE_NAME,
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.SELECTORS_TABLE_NAME,
            name: 'selector',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        selector_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.SELECTORS_TABLE_NAME,
            name: 'item',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        item_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.SELECTORS_TABLE_NAME,
            name: 'query',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        query_id: 'id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'selectors',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.SELECTORS_TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'item_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'selected',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.SELECTORS_TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'selector_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
};
exports.up = up;
const down = async () => {
    log('down');
    log('view');
    await _1616701513782_links_js_1.api.query({
        type: 'untrack_table',
        args: {
            table: {
                schema: _1616701513782_links_js_1.SCHEMA,
                name: exports.SELECTORS_TABLE_NAME,
            },
            cascade: true,
        },
    });
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP VIEW IF EXISTS ${exports.SELECTORS_TABLE_NAME} CASCADE;
  `);
    await _1616701513782_links_js_1.api.query({
        type: 'drop_computed_field',
        args: {
            table: exports.TABLE_NAME,
            source: 'default',
            name: 'exec_bool_exp',
            cascade: false,
        },
    });
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP FUNCTION IF EXISTS ${exports.BOOL_EXP_COMPUTED_FIELD} CASCADE;
  `);
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP FUNCTION IF EXISTS bool_exp_execute CASCADE;
  `);
};
exports.down = down;
//# sourceMappingURL=1622421760258-selectors.js.map