"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.MP = exports.CACHE = exports.TABLE_NAME = exports.CAN_TABLE_NAME = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const sql_js_1 = require("@deep-foundation/hasura/sql.js");
const debug_1 = __importDefault(require("debug"));
const client_js_2 = require("../imports/client.js");
const _1616701513782_links_js_1 = require("./1616701513782-links.js");
const debug = (0, debug_1.default)('deeplinks:migrations:can');
const log = debug.extend('log');
const error = debug.extend('error');
exports.CAN_TABLE_NAME = 'can';
exports.TABLE_NAME = 'links';
exports.CACHE = 'selectors_cache';
exports.MP = 'mp';
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({
    apolloClient: client,
});
const up = async () => {
    log('up');
    log('view');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    CREATE VIEW ${exports.CAN_TABLE_NAME} AS
    SELECT DISTINCT ca."rule_id" as "rule_id", mpo_include."item_id" as "object_id", mps_include."item_id" as "subject_id", mpa_include."item_id" as "action_id"
    FROM
    ${exports.CACHE} as ca,
    ${exports.MP} as mpa_include,
    ${exports.CACHE} as co,
    ${exports.MP} as mpo_include,
    ${exports.CACHE} as cs,
    ${exports.MP} as mps_include
    WHERE (
      ca."rule_id" != 0 AND
      co."rule_id" != 0 AND
      cs."rule_id" != 0 AND

      ca."rule_id" = co."rule_id" AND
      ca."rule_id" = cs."rule_id" AND

      ca."rule_action_id" != 0 AND
      ca."selector_include_id" != 0 AND
      mpa_include."path_item_id" = ca."link_id" AND
      mpa_include."group_id" = ca."tree_id" AND
      NOT EXISTS (
        SELECT mpa_exclude."id"
        FROM
        ${exports.CACHE} as cache_exclude,
        ${exports.MP} as mpa_exclude
        WHERE (
          cache_exclude."selector_exclude_id" != 0 AND
          cache_exclude."selector_id" = ca."selector_id" AND
          mpa_exclude."path_item_id" = cache_exclude."link_id" AND
          mpa_exclude."item_id" = mpa_include."item_id" AND
          mpa_exclude."group_id" = cache_exclude."tree_id"
        )
      ) AND

      co."rule_object_id" != 0 AND
      co."selector_include_id" != 0 AND
      mpo_include."path_item_id" = co."link_id" AND
      mpo_include."group_id" = co."tree_id" AND
      NOT EXISTS (
        SELECT mpo_exclude."id"
        FROM
        ${exports.CACHE} as cache_exclude,
        ${exports.MP} as mpo_exclude
        WHERE (
          cache_exclude."selector_exclude_id" != 0 AND
          cache_exclude."selector_id" = co."selector_id" AND
          mpo_exclude."path_item_id" = cache_exclude."link_id" AND
          mpo_exclude."item_id" = mpo_include."item_id" AND
          mpo_exclude."group_id" = cache_exclude."tree_id"
        )
      ) AND

      cs."rule_subject_id" != 0 AND
      cs."selector_include_id" != 0 AND
      mps_include."path_item_id" = cs."link_id" AND
      mps_include."group_id" = cs."tree_id" AND
      NOT EXISTS (
        SELECT mps_exclude."id"
        FROM
        ${exports.CACHE} as cache_exclude,
        ${exports.MP} as mps_exclude
        WHERE (
          cache_exclude."selector_exclude_id" != 0 AND
          cache_exclude."selector_id" = cs."selector_id" AND
          mps_exclude."path_item_id" = cache_exclude."link_id" AND
          mps_exclude."item_id" = mps_include."item_id" AND
          mps_exclude."group_id" = cache_exclude."tree_id"
        )
      )
    );
  `);
    await _1616701513782_links_js_1.api.query({
        type: 'track_table',
        args: {
            schema: _1616701513782_links_js_1.SCHEMA,
            name: exports.CAN_TABLE_NAME,
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.CAN_TABLE_NAME,
            name: 'rule',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        rule_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.CAN_TABLE_NAME,
            name: 'object',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        object_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.CAN_TABLE_NAME,
            name: 'subject',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        subject_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_object_relationship',
        args: {
            table: exports.CAN_TABLE_NAME,
            name: 'action',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.TABLE_NAME,
                    },
                    column_mapping: {
                        action_id: 'id',
                    },
                    insertion_order: 'before_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'can_object',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.CAN_TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'object_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'can_subject',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.CAN_TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'subject_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'can_action',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.CAN_TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'action_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    await _1616701513782_links_js_1.api.query({
        type: 'create_array_relationship',
        args: {
            table: exports.TABLE_NAME,
            name: 'can_rule',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: _1616701513782_links_js_1.SCHEMA,
                        name: exports.CAN_TABLE_NAME,
                    },
                    column_mapping: {
                        id: 'rule_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
};
exports.up = up;
const down = async () => {
    log('down');
    log('view');
    await _1616701513782_links_js_1.api.query({
        type: 'untrack_table',
        args: {
            table: {
                schema: _1616701513782_links_js_1.SCHEMA,
                name: exports.CAN_TABLE_NAME,
            },
            cascade: true,
        },
    });
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP VIEW IF EXISTS ${exports.CAN_TABLE_NAME} CASCADE;
  `);
};
exports.down = down;
//# sourceMappingURL=1622421760259-can.js.map