"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.linksPermissions = exports.isAdminBoolExp = exports.REPLACE_PATTERN_ID = exports.TABLE_NAME = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const sql_js_1 = require("@deep-foundation/hasura/sql.js");
const debug_1 = __importDefault(require("debug"));
const client_js_2 = require("../imports/client.js");
const permission_js_1 = require("../imports/permission.js");
const _1616701513782_links_js_1 = require("./1616701513782-links.js");
const _1621815803572_materialized_path_js_1 = require("./1621815803572-materialized-path.js");
const _1622421760258_selectors_js_1 = require("./1622421760258-selectors.js");
const _1622421760259_can_js_1 = require("./1622421760259-can.js");
const debug = (0, debug_1.default)('deeplinks:migrations:permissions');
const log = debug.extend('log');
const error = debug.extend('error');
exports.TABLE_NAME = 'links';
exports.REPLACE_PATTERN_ID = '777777777777';
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({
    apolloClient: client,
});
const isAdminBoolExp = async (subjectId = 'X-Hasura-User-Id') => ({
    "_table": {
        "schema": "public",
        "name": "can"
    },
    _where: {
        object_id: { _eq: subjectId },
        subject_id: { _eq: subjectId },
        action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowAdmin') },
    },
});
exports.isAdminBoolExp = isAdminBoolExp;
const linksPermissions = async (self, subjectId = 'X-Hasura-User-Id', role) => ({
    role,
    select: {
        _or: [
            {
                type: {
                    can_object: {
                        action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowSelectType') },
                        subject_id: { _eq: subjectId },
                    },
                }
            },
            {
                can_object: {
                    action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowSelect') },
                    subject_id: { _eq: subjectId },
                },
            },
        ],
    },
    insert: {
        type: {},
        _or: [
            {
                type: {
                    can_object: {
                        action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowInsertType') },
                        subject_id: { _eq: subjectId },
                    },
                },
            },
        ]
    },
    update: {
        _or: [
            {
                can_object: {
                    action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowUpdate') },
                    subject_id: { _eq: subjectId },
                },
            },
            {
                type: {
                    can_object: {
                        action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowUpdateType') },
                        subject_id: { _eq: subjectId },
                    },
                },
            },
        ]
    },
    delete: {
        _or: [
            {
                can_object: {
                    action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowDelete') },
                    subject_id: { _eq: subjectId },
                },
            },
            {
                type: {
                    can_object: {
                        action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowDeleteType') },
                        subject_id: { _eq: subjectId },
                    },
                },
            },
        ]
    },
    columns: ['id', 'from_id', 'to_id', 'type_id'],
    computed_fields: ['value'],
});
exports.linksPermissions = linksPermissions;
const up = async () => {
    log('up');
    log('hasura permissions');
    // const isAdminBoolExp = { _by_item: {
    //   group_id: { _eq: await deep.id('@deep-foundation/core', 'joinTree') },
    //   path_item: {
    //     to: {
    //       type_id: { _eq: await deep.id('@deep-foundation/core', 'Contain') },
    //       from_id: { _eq: await deep.id('deep') },
    //       string: { value: { _eq: "admin" } },
    //     }
    //   }
    // } };
    // const isAdminBoolExp = async (subjectId = 'X-Hasura-User-Id') => ({
    //   object_id: { _eq: subjectId },
    //   subject_id: { _eq: subjectId },
    //   action_id: { _eq: await deep.id('@deep-foundation/core', 'AllowAdmin') },
    // });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1621815803572_materialized_path_js_1.MP_TABLE_NAME, {
        role: 'link',
        select: {
            _and: [
                {
                    item: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).select,
                },
                {
                    path_item: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).select,
                },
            ]
        },
        insert: {
            id: { _is_null: true }
        },
        update: {
            id: { _is_null: true }
        },
        delete: {
            id: { _is_null: true }
        },
        columns: '*',
        computed_fields: [],
    });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1621815803572_materialized_path_js_1.MP_TABLE_NAME, {
        role: 'undefined',
        select: {
            id: { _is_null: true },
        },
        insert: {
            id: { _is_null: true }
        },
        update: {
            id: { _is_null: true }
        },
        delete: {
            id: { _is_null: true }
        },
        columns: '*',
        computed_fields: [],
    });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1621815803572_materialized_path_js_1.TREE_TABLE_NAME, {
        role: 'link',
        select: {
            _and: [
                {
                    link: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).select,
                },
                {
                    parent: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).select,
                },
            ]
        },
        insert: {
            id: { _is_null: true }
        },
        update: {
            id: { _is_null: true }
        },
        delete: {
            id: { _is_null: true }
        },
        columns: '*',
        computed_fields: [],
    });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1621815803572_materialized_path_js_1.TREE_TABLE_NAME, {
        role: 'undefined',
        select: {
            id: { _is_null: true },
        },
        insert: {
            id: { _is_null: true }
        },
        update: {
            id: { _is_null: true }
        },
        delete: {
            id: { _is_null: true }
        },
        columns: '*',
        computed_fields: [],
    });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1622421760259_can_js_1.CAN_TABLE_NAME, {
        role: 'link',
        select: {
        // _and: [
        //   {
        //     subject: (await linksPermissions(['$','link_id'], 'X-Hasura-User-Id', 'link')).select,
        //   },
        //   {
        //     object: (await linksPermissions(['$','link_id'], 'X-Hasura-User-Id', 'link')).select,
        //   },
        //   {
        //     action: (await linksPermissions(['$','link_id'], 'X-Hasura-User-Id', 'link')).select,
        //   },
        //   {
        //     rule: (await linksPermissions(['$','link_id'], 'X-Hasura-User-Id', 'link')).select,
        //   },
        // ]
        },
        insert: {},
        update: {},
        delete: {},
        columns: '*',
        computed_fields: [],
    });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1622421760259_can_js_1.CAN_TABLE_NAME, {
        role: 'undefined',
        select: {
            object_id: { _is_null: true },
            subject_id: { _is_null: true },
            action_id: { _is_null: true },
            rule_id: { _is_null: true },
        },
        insert: {},
        update: {},
        delete: {},
        columns: '*',
        computed_fields: [],
    });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1622421760258_selectors_js_1.SELECTORS_TABLE_NAME, {
        role: 'link',
        select: {
            _and: [
                {
                    selector: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).select,
                },
                {
                    item: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).select,
                },
            ]
        },
        insert: {},
        update: {},
        delete: {},
        columns: '*',
        computed_fields: [],
    });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1622421760258_selectors_js_1.SELECTORS_TABLE_NAME, {
        role: 'undefined',
        select: {
            selector_id: { _is_null: true },
            item_id: { _is_null: true },
        },
        insert: {},
        update: {},
        delete: {},
        columns: '*',
        computed_fields: [],
    });
    await (async () => {
        await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1616701513782_links_js_1.TABLE_NAME, (await (0, exports.linksPermissions)(['$', 'id'], 'X-Hasura-User-Id', 'link')));
        const valuesPermissions = {
            ...(await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')),
            select: {
                link: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).select,
            },
            insert: {
                link: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).insert,
            },
            update: {
                link: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).update,
            },
            delete: {
                link: (await (0, exports.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).delete,
            },
            columns: ['id', 'link_id', 'value'],
            computed_fields: [],
        };
        await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, 'strings', valuesPermissions);
        await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, 'numbers', valuesPermissions);
        await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, 'objects', valuesPermissions);
    })();
    await (async () => {
        await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, _1616701513782_links_js_1.TABLE_NAME, (await (0, exports.linksPermissions)(['$', 'id'], client_js_2._ids?.['@deep-foundation/core']?.Any, 'undefined')));
        const valuesPermissions = {
            ...(await (0, exports.linksPermissions)(['$', 'link_id'], client_js_2._ids?.['@deep-foundation/core']?.Any, 'undefined')),
            select: {
                link: (await (0, exports.linksPermissions)(['$', 'link_id'], client_js_2._ids?.['@deep-foundation/core']?.Any, 'undefined')).select,
            },
            insert: {
                link: (await (0, exports.linksPermissions)(['$', 'link_id'], client_js_2._ids?.['@deep-foundation/core']?.Any, 'undefined')).insert,
            },
            update: {
                link: (await (0, exports.linksPermissions)(['$', 'link_id'], client_js_2._ids?.['@deep-foundation/core']?.Any, 'undefined')).update,
            },
            delete: {
                link: (await (0, exports.linksPermissions)(['$', 'link_id'], client_js_2._ids?.['@deep-foundation/core']?.Any, 'undefined')).delete,
            },
            columns: ['id', 'link_id', 'value'],
            computed_fields: [],
        };
        await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, 'strings', valuesPermissions);
        await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, 'numbers', valuesPermissions);
        await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, 'objects', valuesPermissions);
    })();
    log('postgresql triggers');
    log('insert');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    CREATE OR REPLACE FUNCTION public.${exports.TABLE_NAME}__permissions__insert_links__function()
    RETURNS trigger
    LANGUAGE plpgsql
    AS $function$
      DECLARE
        boolExp RECORD;
        typeLink RECORD;
        fromLink RECORD;
        toLink RECORD;
        sqlResult BOOL;
        session_variables json;
        user_id bigint;
        userRole TEXT;
        foundedBoolExpError RECORD;
        foundedNotErroredBoolExp BOOL = FALSE;
      BEGIN
        session_variables := current_setting('hasura.user', 't');
        IF session_variables IS NULL THEN
          session_variables := ('{ "x-hasura-role": "link", "x-hasura-user-id": "${await deep.id('@deep-foundation/core', 'Any')}" }')::json;
        END IF;
        user_id := (session_variables::json->>'x-hasura-user-id')::bigint;
        userRole := (session_variables::json->>'x-hasura-role')::text;

        IF (NEW."from_id" != NEW."to_id" AND (NEW."from_id" = 0 OR NEW."to_id" = 0)) THEN
          RAISE EXCEPTION 'Particular links is not allowed id: %, from: %, to: %, type: %.', NEW."id", NEW."from_id", NEW."to_id", NEW."type_id";
        END IF;

        SELECT t.* into typeLink
        FROM "${exports.TABLE_NAME}" as t
        WHERE t."id" = NEW."type_id";

        IF (typeLink IS NULL AND NOT (NEW."type_id" = 0 AND userRole = 'admin')) THEN
          RAISE EXCEPTION 'Type % not founded.', NEW."type_id";
        END IF;

        SELECT t.* into fromLink
        FROM "${exports.TABLE_NAME}" as t
        WHERE t."id" = NEW."from_id";

        SELECT t.* into toLink
        FROM "${exports.TABLE_NAME}" as t
        WHERE t."id" = NEW."to_id";

        IF (NEW."from_id" != 0 AND NEW."to_id" != 0) THEN
          IF (typeLink."from_id" != ${client_js_2._ids?.['@deep-foundation/core']?.Any} AND typeLink."from_id" != fromLink."type_id") THEN
            RAISE EXCEPTION 'Type conflict link: { id: %, type: %, from: %, to: % } expected type: { type: %, from: %, to: % } received type: { type: %, from: %, to: % }',
              NEW."id", NEW."type_id", NEW."from_id", NEW."to_id",
              typeLink."id", typeLink."from_id", typeLink."to_id",
              typeLink."id", fromLink."type_id", toLink."type_id"
            ;
          END IF;
          IF (typeLink."to_id" != ${client_js_2._ids?.['@deep-foundation/core']?.Any} AND typeLink."to_id" != toLink."type_id") THEN
            RAISE EXCEPTION 'Type conflict link: { id: %, type: %, from: %, to: % } expected type: { type: %, from: %, to: % } received type: { type: %, from: %, to: % }',
              NEW."id", NEW."type_id", NEW."from_id", NEW."to_id",
              typeLink."id", typeLink."from_id", typeLink."to_id",
              typeLink."id", fromLink."type_id", toLink."type_id"
            ;
          END IF;
        END IF;

        IF user_id IS NOT NULL AND userRole = 'link' THEN
          FOR boolExp
          IN (
            SELECT sr.*
            FROM
            "${_1622421760259_can_js_1.CAN_TABLE_NAME}" as can,
            "${exports.TABLE_NAME}" as ro,
            "${_1622421760258_selectors_js_1.SELECTORS_TABLE_NAME}" as sr
            WHERE
            can."object_id" = NEW."type_id" AND
            can."subject_id" = user_id AND
            can."action_id" = ${await deep.id('@deep-foundation/core', 'AllowInsertType')} AND
            ro."type_id" = ${await deep.id('@deep-foundation/core', 'RuleObject')} AND
            ro."from_id" = can."rule_id" AND
            sr."selector_id" = ro."to_id" AND
            sr."item_id" = NEW."type_id" AND
            sr."query_id" != 0
          )
          LOOP
            SELECT INTO sqlResult bool_exp_execute(NEW.id, boolExp."query_id", user_id);
            IF (sqlResult = FALSE) THEN
              foundedBoolExpError := boolExp;
            END IF;
            IF (sqlResult IS NULL) THEN
              RAISE EXCEPTION 'insert % rejected because selector_id: % query_id: % user_id: % return null', json_agg(NEW), boolExp."selector_id", boolExp."query_id", user_id;
            END IF;
            IF (sqlResult = TRUE) THEN
              foundedNotErroredBoolExp := TRUE;
            END IF;
          END LOOP;
        END IF;

        IF foundedBoolExpError IS NOT NULL AND foundedNotErroredBoolExp = FALSE THEN
          RAISE EXCEPTION 'insert % rejected because selector_id: % query_id: % user_id: % return false', json_agg(NEW), foundedBoolExpError."selector_id", foundedBoolExpError."query_id", user_id;
        END IF;

        RETURN NEW;
      END;
    $function$;
    CREATE TRIGGER ${exports.TABLE_NAME}__permissions__insert_links__trigger AFTER INSERT ON "${exports.TABLE_NAME}" FOR EACH ROW EXECUTE PROCEDURE ${exports.TABLE_NAME}__permissions__insert_links__function();
  `);
    log('delete');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    CREATE OR REPLACE FUNCTION public.${exports.TABLE_NAME}__permissions__delete_links__function()
    RETURNS trigger
    LANGUAGE plpgsql
    AS $function$
      DECLARE
        boolExp RECORD;
        sqlResult BOOL;
        session_variables json;
        user_id bigint;
        userRole TEXT;
        foundedBoolExpError RECORD;
        foundedNotErroredBoolExp BOOL = FALSE;
      BEGIN

      session_variables := current_setting('hasura.user', 't');
      IF session_variables IS NULL THEN
        session_variables := ('{ "x-hasura-role": "link", "x-hasura-user-id": "${await deep.id('@deep-foundation/core', 'Any')}" }')::json;
      END IF;
      user_id := (session_variables::json->>'x-hasura-user-id')::bigint;
      userRole := (session_variables::json->>'x-hasura-role')::text;

      IF user_id IS NOT NULL AND userRole = 'link' THEN
        FOR boolExp
        IN (
          SELECT sr.*
          FROM
          "${_1622421760259_can_js_1.CAN_TABLE_NAME}" as can,
          "${exports.TABLE_NAME}" as ro,
          "${_1622421760258_selectors_js_1.SELECTORS_TABLE_NAME}" as sr
          WHERE
          can."object_id" = OLD."type_id" AND
          can."subject_id" = user_id AND
          can."action_id" = ${await deep.id('@deep-foundation/core', 'AllowDeleteType')} AND
          ro."type_id" = ${await deep.id('@deep-foundation/core', 'RuleObject')} AND
          ro."from_id" = can."rule_id" AND
          sr."selector_id" = ro."to_id" AND
          sr."item_id" = OLD."type_id" AND
          sr."query_id" IS NOT NULL AND
          sr."query_id" != 0
        )
        LOOP
          SELECT INTO sqlResult bool_exp_execute(OLD.id, boolExp."query_id", user_id);
          IF (sqlResult = FALSE) THEN
            foundedBoolExpError := boolExp;
          END IF;
          IF (sqlResult IS NULL) THEN
            RAISE EXCEPTION 'delete % rejected because selector_id: % query_id: % user_id: % return null', json_agg(OLD), boolExp."selector_id", boolExp."query_id", user_id;
          END IF;
          IF (sqlResult = TRUE) THEN
            foundedNotErroredBoolExp := TRUE;
          END IF;
        END LOOP;
      END IF;

      IF foundedBoolExpError IS NOT NULL AND foundedNotErroredBoolExp = FALSE THEN
        RAISE EXCEPTION 'delete % rejected because selector_id: % query_id: % user_id: % return false', json_agg(OLD), foundedBoolExpError."selector_id", foundedBoolExpError."query_id", user_id;
      END IF;

      RETURN OLD;
      END;
    $function$;
    CREATE TRIGGER ${exports.TABLE_NAME}__permissions__delete_links__trigger BEFORE DELETE ON "${exports.TABLE_NAME}" FOR EACH ROW EXECUTE PROCEDURE ${exports.TABLE_NAME}__permissions__delete_links__function();
  `);
};
exports.up = up;
const down = async () => {
    log('down');
    log('insert');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP TRIGGER IF EXISTS ${exports.TABLE_NAME}__permissions__insert_links__trigger ON ${exports.TABLE_NAME} CASCADE;
    DROP FUNCTION IF EXISTS ${exports.TABLE_NAME}__permissions__insert_links__function() CASCADE;
  `);
    log('delete');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP TRIGGER IF EXISTS ${exports.TABLE_NAME}__permissions__delete_links__trigger ON ${exports.TABLE_NAME} CASCADE;
    DROP FUNCTION IF EXISTS ${exports.TABLE_NAME}__permissions__delete_links__function() CASCADE;
  `);
};
exports.down = down;
//# sourceMappingURL=1622421760260-permissions.js.map