"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = void 0;
const api_js_1 = require("@deep-foundation/hasura/api.js");
const client_js_1 = require("@deep-foundation/hasura/client.js");
const debug_1 = __importDefault(require("debug"));
const client_js_2 = require("../imports/client.js");
const packager_js_1 = require("../imports/router/packager.js");
const debug = (0, debug_1.default)('deeplinks:migrations:packager');
const log = debug.extend('log');
const error = debug.extend('error');
const api = new api_js_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({
    apolloClient: client,
});
const up = async () => {
    log('up');
    await api.query({
        type: 'add_remote_schema',
        args: {
            name: 'packager',
            definition: {
                url: `${process.env.MIGRATIONS_DEEPLINKS_URL}/api/packager`,
                headers: [{ name: 'x-hasura-client', value: 'deeplinks-packager' }],
                forward_client_headers: true,
                timeout_seconds: 60
            },
        }
    });
    await api.metadata({
        type: "add_remote_schema_permissions",
        args: {
            remote_schema: 'packager',
            role: 'link',
            definition: {
                schema: packager_js_1.typeDefsString,
            },
        },
    });
};
exports.up = up;
const down = async () => {
    log('down');
    await api.query({
        type: 'remove_remote_schema',
        args: {
            name: 'packager',
        },
    });
};
exports.down = down;
//# sourceMappingURL=1637975150573-packager.js.map