"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = void 0;
const api_js_1 = require("@deep-foundation/hasura/api.js");
const debug_1 = __importDefault(require("debug"));
const authorization_js_1 = require("../imports/router/authorization.js");
const debug = (0, debug_1.default)('deeplinks:migrations:authorization');
const log = debug.extend('log');
const error = debug.extend('error');
const api = new api_js_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const up = async () => {
    log('up');
    await api.query({
        type: 'add_remote_schema',
        args: {
            name: 'authorization',
            definition: {
                url: `${process.env.MIGRATIONS_DEEPLINKS_URL}/api/authorization`,
                headers: [{ name: 'x-hasura-client', value: 'deeplinks-authorization' }],
                forward_client_headers: true,
                timeout_seconds: 120
            },
        }
    });
    await api.metadata({
        type: "add_remote_schema_permissions",
        args: {
            remote_schema: 'authorization',
            role: 'link',
            definition: {
                schema: authorization_js_1.typeDefsString,
            },
        },
    });
    await api.metadata({
        type: "add_remote_schema_permissions",
        args: {
            remote_schema: 'authorization',
            role: 'undefined',
            definition: {
                schema: authorization_js_1.typeDefsString,
            },
        },
    });
};
exports.up = up;
const down = async () => {
    log('down');
    await api.query({
        type: 'remove_remote_schema',
        args: {
            name: 'authorization',
        },
    });
};
exports.down = down;
//# sourceMappingURL=1657486935281-authorization.js.map