"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.TABLE_NAME = exports.HANDLERS_TABLE_NAME = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const sql_js_1 = require("@deep-foundation/hasura/sql.js");
const debug_1 = __importDefault(require("debug"));
const client_js_2 = require("../imports/client.js");
const permission_js_1 = require("../imports/permission.js");
const _1616701513782_links_js_1 = require("./1616701513782-links.js");
const _1622421760260_permissions_js_1 = require("./1622421760260-permissions.js");
const debug = (0, debug_1.default)('deeplinks:migrations:handlers');
const log = debug.extend('log');
const error = debug.extend('error');
exports.HANDLERS_TABLE_NAME = 'handlers';
exports.TABLE_NAME = 'links';
const client = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const deep = new client_js_2.DeepClient({
    apolloClient: client,
});
const up = async () => {
    log('up');
    log('view');
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    CREATE VIEW ${exports.HANDLERS_TABLE_NAME} AS
    SELECT DISTINCT
      coalesce((SELECT gl.from_id as id FROM links gl WHERE (gl.type_id = ${await deep.id('@deep-foundation/core', 'GeneratedFrom')}) AND (gl.from_id = handling_link.id OR gl.to_id = handling_link.id) LIMIT 1), handling_link.id) AS dist_id,
      coalesce((SELECT gl.to_id as id FROM links gl WHERE (gl.type_id = ${await deep.id('@deep-foundation/core', 'GeneratedFrom')}) AND (gl.from_id = handling_link.id OR gl.to_id = handling_link.id) LIMIT 1), handling_link.id) AS src_id,
      handler_link.id AS handler_id,
      supports_link.from_id AS isolation_provider_id,
      supports_link.to_id AS execution_provider_id
    FROM links handling_link,
      links handler_link,
      links supports_link
    WHERE (
      (handler_link.type_id = ${await deep.id('@deep-foundation/core', 'Handler')})
      AND (handler_link.from_id = supports_link.id)
      AND (supports_link.type_id = ${await deep.id('@deep-foundation/core', 'Supports')})
      AND (handler_link.to_id = handling_link.id)
    );
  `);
    await _1616701513782_links_js_1.api.query({
        type: 'track_table',
        args: {
            schema: _1616701513782_links_js_1.SCHEMA,
            name: exports.HANDLERS_TABLE_NAME,
        },
    });
    const relationsFields = ['src', 'dist', 'handler', 'isolation_provider', 'execution_provider'];
    for (const field of relationsFields) {
        await _1616701513782_links_js_1.api.query({
            type: 'create_object_relationship',
            args: {
                table: exports.HANDLERS_TABLE_NAME,
                name: field,
                using: {
                    manual_configuration: {
                        remote_table: {
                            schema: _1616701513782_links_js_1.SCHEMA,
                            name: exports.TABLE_NAME,
                        },
                        column_mapping: {
                            [`${field}_id`]: 'id',
                        },
                    },
                },
            },
        });
    }
    const permissionFields = ['dist'];
    const selectPermissionsLinks = [];
    for (const field of permissionFields) {
        selectPermissionsLinks.push({
            [field]: (await (0, _1622421760260_permissions_js_1.linksPermissions)(['$', 'link_id'], 'X-Hasura-User-Id', 'link')).select,
        });
    }
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, exports.HANDLERS_TABLE_NAME, {
        role: 'link',
        select: {
            _and: selectPermissionsLinks,
        },
        insert: {},
        update: {},
        delete: {},
        columns: '*',
        computed_fields: [],
    });
    await (0, permission_js_1.permissions)(_1616701513782_links_js_1.api, exports.HANDLERS_TABLE_NAME, {
        role: 'undefined',
        select: {
            dist_id: { _is_null: true },
        },
        insert: {},
        update: {},
        delete: {},
        columns: '*',
        computed_fields: [],
    });
};
exports.up = up;
const down = async () => {
    log('down');
    log('view');
    await _1616701513782_links_js_1.api.query({
        type: 'untrack_table',
        args: {
            table: {
                schema: _1616701513782_links_js_1.SCHEMA,
                name: exports.HANDLERS_TABLE_NAME,
            },
            cascade: true,
        },
    });
    await _1616701513782_links_js_1.api.sql((0, sql_js_1.sql) `
    DROP VIEW IF EXISTS ${exports.HANDLERS_TABLE_NAME} CASCADE;
  `);
};
exports.down = down;
//# sourceMappingURL=1657720948151-handlers.js.map