"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.TABLE_NAME = exports.SCHEMA = exports.api = void 0;
const api_js_1 = require("@deep-foundation/hasura/api.js");
const sql_js_1 = require("@deep-foundation/hasura/sql.js");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:migrations:links');
const log = debug.extend('log');
const error = debug.extend('error');
exports.api = new api_js_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
exports.SCHEMA = 'public';
exports.TABLE_NAME = 'links';
const up = async () => {
    log('up');
    await exports.api.sql((0, sql_js_1.sql) `
    CREATE OR REPLACE FUNCTION public.${exports.TABLE_NAME}__unvalue__delete_links__function()
    RETURNS trigger
    LANGUAGE plpgsql
    AS $function$
      BEGIN
        DELETE FROM strings WHERE "link_id" = OLD."id";
        DELETE FROM numbers WHERE "link_id" = OLD."id";
        DELETE FROM objects WHERE "link_id" = OLD."id";
        RETURN OLD;
      END;
    $function$;
    CREATE TRIGGER ${exports.TABLE_NAME}__unvalue__delete_links__trigger AFTER DELETE ON "${exports.TABLE_NAME}" FOR EACH ROW EXECUTE PROCEDURE ${exports.TABLE_NAME}__unvalue__delete_links__function();
  `);
};
exports.up = up;
const down = async () => {
    log('down');
    await exports.api.sql((0, sql_js_1.sql) `
    DROP TRIGGER IF EXISTS ${exports.TABLE_NAME}__unvalue__delete_links__trigger ON ${exports.TABLE_NAME} CASCADE;
    DROP FUNCTION IF EXISTS ${exports.TABLE_NAME}__unvalue__delete_links__function() CASCADE;
  `);
};
exports.down = down;
//# sourceMappingURL=1658622099992-unvalue.js.map