"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.containWithin = exports.sharePermissions = exports.packageExists = exports.importPackage = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const debug_1 = __importDefault(require("debug"));
const client_js_2 = require("../imports/client.js");
const deep_json_1 = __importDefault(require("@deep-foundation/tsx/deep.json"));
const packager_js_1 = require("../imports/packager.js");
const debug = (0, debug_1.default)('deeplinks:migrations:tsx');
const log = debug.extend('log');
const error = debug.extend('error');
const rootClient = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const root = new client_js_2.DeepClient({
    apolloClient: rootClient,
});
const importPackage = async (pckg) => {
    const packager = new packager_js_1.Packager(root);
    const importResult = await packager.import(pckg);
    const { errors, packageId, namespaceId } = importResult;
    if (errors?.length) {
        console.log(JSON.stringify(errors, null, 2));
        const error = errors[0]?.graphQLErrors?.[0]?.extensions?.internal?.error;
        throw new Error(`Import error: ${String(errors[0]?.graphQLErrors?.[0]?.message || errors?.[0])}${error?.message ? ` ${error?.message} ${error?.request?.method} ${error?.request?.host}:${error?.request?.port}${error?.request?.path}` : ''}`);
    }
    return importResult;
};
exports.importPackage = importPackage;
const packageExists = async (name) => {
    try {
        const result = (await root.id(name)) > 0;
        // console.log('packageExists', 'result', result);
        return result;
    }
    catch {
        // console.log('packageExists', 'result', false);
        return false;
    }
};
exports.packageExists = packageExists;
const sharePermissions = async (userId, packageId) => {
    return await root.insert({
        type_id: await root.id('@deep-foundation/core', 'Join'),
        from_id: packageId,
        to_id: userId,
    });
};
exports.sharePermissions = sharePermissions;
const containWithin = async (containerId, containedId) => {
    return await root.insert({
        type_id: await root.id('@deep-foundation/core', 'Contain'),
        from_id: containerId,
        to_id: containedId,
    });
};
exports.containWithin = containWithin;
const up = async () => {
    log('up');
    if (!await (0, exports.packageExists)('@deep-foundation/tsx')) {
        const importResult = await (0, exports.importPackage)(deep_json_1.default);
        log(importResult);
        const packageId = importResult.packageId;
        const adminId = await root.id('deep', 'admin');
        await (0, exports.sharePermissions)(adminId, packageId);
        await (0, exports.containWithin)(adminId, packageId);
    }
};
exports.up = up;
const down = async () => {
    log('down');
};
exports.down = down;
//# sourceMappingURL=1664940577200-tsx.js.map