"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = exports.installPackage = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const debug_1 = __importDefault(require("debug"));
const client_js_2 = require("../imports/client.js");
const _1664940577200_tsx_js_1 = require("./1664940577200-tsx.js");
const debug = (0, debug_1.default)('deeplinks:migrations:deepcase');
const log = debug.extend('log');
const error = debug.extend('error');
const rootClient = (0, client_js_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const root = new client_js_2.DeepClient({
    apolloClient: rootClient,
});
const installPackage = async (deep, packageName) => {
    const adminId = await deep.id('deep', 'admin');
    log('adminId', adminId);
    const packageQueryTypeId = await deep.id('@deep-foundation/core', 'PackageQuery');
    log('packageQueryTypeId', packageQueryTypeId);
    const installTypeId = await deep.id('@deep-foundation/npm-packager', 'Install');
    log('installTypeId', installTypeId);
    const { data: [{ id: installId = undefined } = {}] = [] } = {} = await deep.insert({
        from_id: adminId,
        type_id: installTypeId,
        to: {
            data: {
                type_id: packageQueryTypeId,
                string: { data: { value: packageName } }
            }
        }
    });
    await deep.await(installId);
    const packageId = await deep.id(packageName);
    log(`${packageName} package is installed as ${packageId} link.`);
    return packageId;
};
exports.installPackage = installPackage;
const up = async () => {
    log('up');
    const packageName = '@deep-foundation/deepcase';
    if (!await (0, _1664940577200_tsx_js_1.packageExists)(packageName)) {
        const adminId = await root.id('deep', 'admin');
        const admin = await root.login({ linkId: adminId });
        const deep = new client_js_2.DeepClient({ deep: root, ...admin });
        log('adminId', adminId);
        const packageId = await (0, exports.installPackage)(deep, packageName);
        await (0, _1664940577200_tsx_js_1.sharePermissions)(adminId, packageId);
        const usersCanInsertSafeLinks = await deep.id('deep', 'admin', 'usersCanInsertSafeLinks');
        const usersCanUpdateSafeLinks = await deep.id('deep', 'admin', 'usersCanUpdateSafeLinks');
        const usersCanDeleteSafeLinks = await deep.id('deep', 'admin', 'usersCanDeleteSafeLinks');
        const { data: rules } = await deep.select({
            'up': {
                'parent_id': { _in: [usersCanInsertSafeLinks, usersCanUpdateSafeLinks, usersCanDeleteSafeLinks] },
            }
        });
        deep.minilinks.apply([...rules]);
        const insertSelector = deep.minilinks.byId[usersCanInsertSafeLinks]?.outByType?.[deep.idLocal('@deep-foundation/core', 'RuleObject')]?.[0]?.to?.id;
        const updateSelector = deep.minilinks.byId[usersCanUpdateSafeLinks]?.outByType?.[deep.idLocal('@deep-foundation/core', 'RuleObject')]?.[0]?.to?.id;
        const deleteSelector = deep.minilinks.byId[usersCanDeleteSafeLinks]?.outByType?.[deep.idLocal('@deep-foundation/core', 'RuleObject')]?.[0]?.to?.id;
        await deep.insert([
            {
                type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                from_id: insertSelector,
                to_id: await deep.id(packageName, 'Traveler'),
                out: { data: {
                        type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                        to_id: await deep.id('@deep-foundation/core', 'containTree'),
                    } },
            },
            {
                type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                from_id: updateSelector,
                to_id: await deep.id(packageName, 'Traveler'),
                out: { data: {
                        type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                        to_id: await deep.id('@deep-foundation/core', 'containTree'),
                    } },
            },
            {
                type_id: await deep.id('@deep-foundation/core', 'SelectorInclude'),
                from_id: deleteSelector,
                to_id: await deep.id(packageName, 'Traveler'),
                out: { data: {
                        type_id: await deep.id('@deep-foundation/core', 'SelectorTree'),
                        to_id: await deep.id('@deep-foundation/core', 'containTree'),
                    } },
            },
        ]);
    }
};
exports.up = up;
const down = async () => {
    log('down');
};
exports.down = down;
//# sourceMappingURL=1678940577209-deepcase.js.map