"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const api_js_1 = require("@deep-foundation/hasura/api.js");
const client_js_1 = require("@deep-foundation/hasura/client.js");
const chai_1 = require("chai");
const bool_exp_to_sql_1 = require("../imports/bool_exp_to_sql");
const client_1 = require("../imports/client");
const promise_1 = require("../imports/promise");
exports.api = new api_js_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const apolloClient = (0, client_js_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_1.DeepClient({ apolloClient });
let adminToken;
let admin;
beforeAll(async () => {
    const { linkId, token } = await deep.jwt({ linkId: await deep.id('deep', 'admin') });
    adminToken = token;
    admin = new client_1.DeepClient({ deep, token: adminToken, linkId });
});
describe('bool_exp', () => {
    describe('value convertation', () => {
        it(`insert separately`, async () => {
            const { data: [{ id: t1 }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Any'),
            });
            const { data: [{ id: t2 }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Any'),
            });
            const { data: [{ id: boolExpId }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Query'),
            });
            await deep.insert({
                link_id: boolExpId,
                value: { id: { _eq: t1 } },
            }, { table: 'objects' });
            await deep.await(boolExpId);
            await (0, promise_1.delay)(2000);
            const { data: [{ value: sql }], error } = await deep.select({
                link_id: { _eq: boolExpId },
            }, { table: 'bool_exp', returning: 'id link_id value' });
            const { data: d1 } = await exports.api.sql((0, bool_exp_to_sql_1.applyBoolExpToLink)(sql, t1));
            (0, chai_1.expect)(+d1?.result?.[1]?.[0]).to.equal(1);
            const { data: d2 } = await exports.api.sql((0, bool_exp_to_sql_1.applyBoolExpToLink)(sql, t2));
            (0, chai_1.expect)(+d2?.result?.[1]?.[0]).to.equal(0);
        });
        it(`insert with link`, async () => {
            const { data: [{ id: t1 }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Any'),
            });
            const { data: [{ id: t2 }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Any'),
            });
            const { data: [{ id: boolExpId }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Query'),
                object: { data: { value: { id: { _eq: t1 } } } },
            });
            await deep.await(boolExpId);
            await (0, promise_1.delay)(2000);
            const { data: [{ value: sql }], error } = await deep.select({
                link_id: { _eq: boolExpId },
            }, { table: 'bool_exp', returning: 'id link_id value' });
            const { data: d1 } = await exports.api.sql((0, bool_exp_to_sql_1.applyBoolExpToLink)(sql, t1));
            (0, chai_1.expect)(+d1?.result?.[1]?.[0]).to.equal(1);
            const { data: d2 } = await exports.api.sql((0, bool_exp_to_sql_1.applyBoolExpToLink)(sql, t2));
            (0, chai_1.expect)(+d2?.result?.[1]?.[0]).to.equal(0);
        });
        it(`update`, async () => {
            const { data: [{ id: t1 }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Any'),
            });
            const { data: [{ id: t2 }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Any'),
            });
            const { data: [{ id: boolExpId }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Query'),
                object: { data: { value: { id: { _eq: t1 } } } },
            });
            await deep.await(boolExpId);
            await (0, promise_1.delay)(2000);
            await deep.update({ link_id: { _eq: boolExpId } }, {
                value: { id: { _eq: t2 } },
            }, { table: 'objects' });
            await deep.await(boolExpId);
            await (0, promise_1.delay)(2000);
            const { data: [{ value: sql }], error } = await deep.select({
                link_id: { _eq: boolExpId },
            }, { table: 'bool_exp', returning: 'id link_id value' });
            const { data: d1 } = await exports.api.sql((0, bool_exp_to_sql_1.applyBoolExpToLink)(sql, t1));
            (0, chai_1.expect)(+d1?.result?.[1]?.[0]).to.equal(0);
            const { data: d2 } = await exports.api.sql((0, bool_exp_to_sql_1.applyBoolExpToLink)(sql, t2));
            (0, chai_1.expect)(+d2?.result?.[1]?.[0]).to.equal(1);
        });
        it(`delete`, async () => {
            const { data: [{ id: t1 }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Any'),
            });
            const { data: [{ id: t2 }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Any'),
            });
            const { data: [{ id: boolExpId }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Query'),
                object: { data: { value: { id: { _eq: t1 } } } },
            });
            await deep.await(boolExpId);
            await (0, promise_1.delay)(2000);
            await deep.delete({ link_id: { _eq: boolExpId } }, { table: 'objects' });
            await deep.await(boolExpId);
            await (0, promise_1.delay)(2000);
            const { data, error } = await deep.select({
                link_id: { _eq: boolExpId },
            }, { table: 'bool_exp', returning: 'id link_id value' });
            (0, chai_1.expect)(data).to.be.empty;
        });
        it(`X-Deep-User-Id`, async () => {
            const g1 = await deep.guest({});
            const { data: [{ id: boolExpId }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Query'),
                object: { data: { value: { id: { _eq: 'X-Deep-User-Id' } } } },
                in: { data: {
                        type_id: await deep.id('@deep-foundation/core', 'Contain'),
                        from_id: g1.linkId,
                    } },
            });
            await deep.await(boolExpId);
            await (0, promise_1.delay)(2000);
            const d1s = await deep.select({
                id: { _eq: boolExpId },
            }, { returning: `exec_bool_exp(args: { link_id: ${g1.linkId} }) { id }` });
            (0, chai_1.expect)(d1s?.data?.[0]?.exec_bool_exp || []).to.be.empty;
            const deep1 = new client_1.DeepClient({ deep, ...g1 });
            const g1s = await deep1.select({
                id: { _eq: boolExpId },
            }, { returning: `exec_bool_exp(args: { link_id: ${g1.linkId} }) { id }` });
            (0, chai_1.expect)(g1s?.data?.[0]?.exec_bool_exp || []).to.not.be.empty;
            (0, chai_1.expect)(g1s?.data?.[0]?.exec_bool_exp?.[0]?.id).to.be.equal(g1.linkId);
        });
        it(`X-Deep-Item-Id`, async () => {
            const { data: [{ id: boolExpId }] } = await deep.insert({
                type_id: await deep.id('@deep-foundation/core', 'Query'),
                object: { data: { value: { id: { _eq: 'X-Deep-Item-Id' } } } },
            });
            await deep.await(boolExpId);
            await (0, promise_1.delay)(2000);
            const d1s = await admin.select({
                id: { _eq: boolExpId },
            }, { returning: `exec_bool_exp(args: { link_id: ${boolExpId} }) { id }` });
            (0, chai_1.expect)(d1s?.data?.[0]?.exec_bool_exp?.[0]?.id).to.be.equal(boolExpId);
        });
    });
});
//# sourceMappingURL=bool_exp.js.map