"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_js_1 = require("@deep-foundation/hasura/client.js");
const client_1 = require("../imports/client");
const chai_1 = require("chai");
const util_1 = require("util");
const gql_1 = require("../imports/gql");
const react_1 = require("@testing-library/react");
const client_2 = require("../imports/client");
const react_2 = __importDefault(require("react"));
const index_js_1 = require("@apollo/client/index.js");
require("@testing-library/jest-dom");
const client_3 = require("../imports/client");
const apollo_client_tokenized_provider_1 = require("@deep-foundation/react-hasura/apollo-client-tokenized-provider");
const react_token_1 = require("../imports/react-token");
const local_1 = require("@deep-foundation/store/local");
const query_1 = require("@deep-foundation/store/query");
const cookies_1 = require("@deep-foundation/store/cookies");
const capacitor_1 = require("@deep-foundation/store/capacitor");
function Main({ options }) {
    return react_2.default.createElement(apollo_client_tokenized_provider_1.ApolloClientTokenizedProvider, { options: options },
        react_2.default.createElement("div", null));
}
const graphQlPath = `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`;
const ssl = !!+process.env.DEEPLINKS_HASURA_SSL;
const secret = process.env.DEEPLINKS_HASURA_SECRET;
const ws = true;
let apolloClient;
let deepClient;
beforeAll(async () => {
    apolloClient = (0, client_js_1.generateApolloClient)({
        path: graphQlPath,
        ssl,
        secret,
        ws
    });
    deepClient = new client_1.DeepClient({ apolloClient });
});
describe('client', () => {
    it(`deep.linkId guest and login`, async () => {
        chai_1.assert.equal(deepClient.linkId, undefined);
        chai_1.assert.notEqual(deepClient.linkId, 0);
        const guest = await deepClient.guest();
        const guestDeep = new client_1.DeepClient({ deep: deepClient, ...guest });
        chai_1.assert.notEqual(guestDeep.linkId, undefined);
        chai_1.assert.notEqual(guestDeep.linkId, 0);
        const guestId = guestDeep.linkId;
        const adminId = await deepClient.id('deep', 'admin');
        const admin = await deepClient.login({ linkId: adminId });
        const deep = new client_1.DeepClient({ deep: deepClient, ...admin });
        chai_1.assert.notEqual(deep.linkId, undefined);
        chai_1.assert.notEqual(deep.linkId, 0);
        chai_1.assert.notEqual(deep.linkId, guestId);
    });
    it('name', async () => {
        const containTypeLinkId = await deepClient.id("@deep-foundation/core", "Contain");
        const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
        const typeName = "MyTypeName";
        const reservedLinkIds = await deepClient.reserve(2);
        const newTypeLinkId = reservedLinkIds.pop();
        const containLinkId = reservedLinkIds.pop();
        await deepClient.serial({
            operations: [
                {
                    type: 'insert',
                    table: 'links',
                    objects: {
                        id: newTypeLinkId,
                        type_id: typeTypeLinkId,
                    }
                },
                {
                    type: 'insert',
                    table: 'links',
                    objects: {
                        id: containLinkId,
                        type_id: containTypeLinkId,
                        from_id: await deepClient.id('deep', 'admin'),
                        to_id: newTypeLinkId,
                    }
                },
                {
                    type: 'insert',
                    table: 'strings',
                    objects: {
                        link_id: containLinkId,
                        value: typeName
                    }
                }
            ]
        });
        chai_1.assert.equal(await deepClient.name(newTypeLinkId), typeName);
    });
    it(`{ id: 5 }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ id: 5 }), { id: { _eq: 5 } });
    });
    it(`{ id: { _eq: 5 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ id: { _eq: 5 } }), { id: { _eq: 5 } });
    });
    it(`{ value: 5 }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ value: 5 }), { number: { value: { _eq: 5 } } });
    });
    it(`{ value: 'a' }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ value: 'a' }), { string: { value: { _eq: 'a' } } });
    });
    it(`{ number: 5 }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ number: 5 }), { number: { value: { _eq: 5 } } });
    });
    it(`{ string: 'a' }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ string: 'a' }), { string: { value: { _eq: 'a' } } });
    });
    it(`{ number: { value: { _eq: 5 } } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ number: { value: { _eq: 5 } } }), { number: { value: { _eq: 5 } } });
    });
    it(`{ string: { value: { _eq: 'a' } } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ string: { value: { _eq: 'a' } } }), { string: { value: { _eq: 'a' } } });
    });
    it(`{ object: { value: { _contains: { a: 'b' } } } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ object: { value: { _contains: { a: 'b' } } } }), { object: { value: { _contains: { a: 'b' } } } });
    });
    it(`{ from: { value: 5 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ from: { value: 5 } }), { from: { number: { value: { _eq: 5 } } } });
    });
    it(`{ out: { type_id: Contain, value: item, from: where } }`, async () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({
            out: {
                type_id: await deepClient.id('@deep-foundation/core', 'Contain'),
                value: 'b',
                from: {
                    type_id: await deepClient.id('@deep-foundation/core', 'Package'),
                    value: 'a',
                },
            },
        }), {
            out: {
                type_id: { _eq: await deepClient.id('@deep-foundation/core', 'Contain') },
                string: { value: { _eq: 'b' } },
                from: {
                    type_id: { _eq: await deepClient.id('@deep-foundation/core', 'Package') },
                    string: { value: { _eq: 'a' } },
                },
            }
        });
    });
    it(`{ value: 5, link: { type_id: 7 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ value: 5, link: { type_id: 7 } }, 'value'), {
            value: { _eq: 5 },
            link: {
                type_id: { _eq: 7 }
            },
        });
    });
    it(`{ type: ['@deep-foundation/core', 'Value'] }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({
            type: ["@deep-foundation/core", "Value"],
        }), {
            type: {
                in: {
                    from: {
                        string: { value: { _eq: "@deep-foundation/core" } },
                        type_id: { _eq: 2 },
                    },
                    string: { value: { _eq: "Value" } },
                    type_id: { _eq: 3 },
                },
            },
        });
    });
    it(`{ type: ['@deep-foundation/core', 'Value'] }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({
            _or: [{
                    type: ["@deep-foundation/core", "Value"],
                }, {
                    type: ["@deep-foundation/core", "User"],
                }]
        }), {
            _or: [{
                    type: {
                        in: {
                            from: {
                                string: { value: { _eq: "@deep-foundation/core" } },
                                type_id: { _eq: 2 },
                            },
                            string: { value: { _eq: "Value" } },
                            type_id: { _eq: 3 },
                        },
                    },
                }, {
                    type: {
                        in: {
                            from: {
                                string: { value: { _eq: "@deep-foundation/core" } },
                                type_id: { _eq: 2 },
                            },
                            string: { value: { _eq: "User" } },
                            type_id: { _eq: 3 },
                        },
                    },
                }]
        });
    });
    it(`id(packageName,contain)`, async () => {
        const id = await deepClient.id('@deep-foundation/core', 'Value');
        chai_1.assert.equal(id, 4);
    });
    it(`{ type_id: { _type_of: 25 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ type_id: { _type_of: 25 } }), { type: { _by_item: { path_item_id: { _eq: 25 }, group_id: { _eq: 0 } } } });
    });
    it(`{ from_id: { _type_of: 25 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ from_id: { _type_of: 25 } }), { from: { _by_item: { path_item_id: { _eq: 25 }, group_id: { _eq: 0 } } } });
    });
    it(`{ to_id: { _type_of: 25 } }`, () => {
        chai_1.assert.deepEqual(deepClient.serializeWhere({ to_id: { _type_of: 25 } }), { to: { _by_item: { path_item_id: { _eq: 25 }, group_id: { _eq: 0 } } } });
    });
    it(`{ id: { _id: ['@deep-foundation/core', 'Package'] } }`, async () => {
        chai_1.assert.deepEqual((await deepClient.select({ id: { _id: ['@deep-foundation/core', 'Package'] } }))?.data?.[0]?.id, 2);
    });
    it(`{ type_id: { _id: ['@deep-foundation/core', 'Package'] } }`, async () => {
        const packageId = await deepClient.id('@deep-foundation/core');
        chai_1.assert.isTrue(!!(await deepClient.select({ type_id: { _id: ['@deep-foundation/core', 'Package'] } }))?.data?.find(p => p.id === packageId));
    });
    it(`{ up: { parent_id: { _id: ['@deep-foundation/core', 'Package'] } } }`, async () => {
        const packageId = await deepClient.id('@deep-foundation/core');
        chai_1.assert.isTrue(!!(await deepClient.select({ up: { parent_id: { _id: ['@deep-foundation/core', 'Package'] } } }))?.data?.find(p => p.id === packageId));
    });
    it(`idLocal get from minilinks`, async () => {
        const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
        const containTypeLinkId = await deepClient.id("@deep-foundation/core", "Contain");
        const packageTypeLinkId = await deepClient.id("@deep-foundation/core", "Package");
        const packageName = "idLocal get from minilinks package";
        const { data: [packageLink] } = await deepClient.insert({
            type_id: packageTypeLinkId,
            string: {
                data: {
                    value: packageName
                }
            }
        }, { returning: deepClient.selectReturning });
        const { data: [newTypeTypeLink] } = await deepClient.insert({
            type_id: typeTypeLinkId,
        }, { returning: deepClient.selectReturning });
        const { data: [containLink] } = await deepClient.insert({
            type_id: containTypeLinkId,
            from_id: packageLink.id,
            to_id: newTypeTypeLink.id,
            string: {
                data: {
                    value: "Type"
                }
            }
        }, { returning: deepClient.selectReturning });
        deepClient.minilinks.apply([packageLink, containLink, newTypeTypeLink]);
        try {
            const packageLinkId = deepClient.idLocal(packageName);
            chai_1.assert.equal(packageLinkId, packageLink.id);
            const newTypeTypeLinkId = deepClient.idLocal(packageName, "Type");
            chai_1.assert.equal(newTypeTypeLinkId, newTypeTypeLink.id);
        }
        finally {
            await deepClient.delete([packageLink.id, newTypeTypeLink.id, containLink.id]);
        }
    });
    it(`insert link with reserved id and string value`, async () => {
        const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
        const reservedIds = await deepClient.reserve(1);
        try {
            await deepClient.insert({
                id: reservedIds[0],
                type_id: typeTypeLinkId,
                string: {
                    data: {
                        value: 'stringValue'
                    }
                }
            });
        }
        finally {
            await deepClient.delete(reservedIds[0]);
        }
    });
    describe(`serial`, () => {
        describe('insert', () => {
            it('one insert', async () => {
                let linkIdsToDelete = [];
                const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
                const operation = (0, gql_1.createSerialOperation)({
                    table: 'links',
                    type: 'insert',
                    objects: {
                        type_id: typeTypeLinkId
                    }
                });
                try {
                    const result = await deepClient.serial({
                        operations: [
                            operation,
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    linkIdsToDelete = [...linkIdsToDelete, ...result.data.map(link => link.id)];
                    chai_1.assert.strictEqual(result.data.length, 1);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
            it('multiple inserts in one operation', async () => {
                let linkIdsToDelete = [];
                const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
                const operation = (0, gql_1.createSerialOperation)({
                    table: 'links',
                    type: 'insert',
                    objects: {
                        type_id: typeTypeLinkId
                    }
                });
                try {
                    const result = await deepClient.serial({
                        operations: [
                            operation,
                            operation
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    linkIdsToDelete = [...linkIdsToDelete, ...result.data.map(link => link.id)];
                    chai_1.assert.strictEqual(result.data.length, 2);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
            it('multiple inserts ine multiple operations', async () => {
                let linkIdsToDelete = [];
                const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
                const operation = (0, gql_1.createSerialOperation)({
                    table: 'links',
                    type: 'insert',
                    objects: {
                        type_id: typeTypeLinkId
                    }
                });
                try {
                    const result = await deepClient.serial({
                        operations: [
                            operation,
                            operation
                        ]
                    });
                    (0, util_1.inspect)(result);
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    linkIdsToDelete = [...linkIdsToDelete, ...result.data.map(link => link.id)];
                    chai_1.assert.strictEqual(result.data.length, 2);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
        });
        describe('update', () => {
            it('one update', async () => {
                let linkIdsToDelete = [];
                const expectedValue = 'newStringValue';
                const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
                try {
                    const insertResult = await deepClient.insert({
                        type_id: typeTypeLinkId,
                        string: {
                            data: {
                                value: "stringValue"
                            }
                        }
                    });
                    chai_1.assert.equal(insertResult.error, undefined);
                    chai_1.assert.notEqual(insertResult.data, undefined);
                    chai_1.assert.strictEqual(insertResult.data.length, 1);
                    const newLinkId = insertResult.data[0].id;
                    linkIdsToDelete.push(newLinkId);
                    const operation = (0, gql_1.createSerialOperation)({
                        table: 'strings',
                        type: 'update',
                        exp: {
                            link_id: newLinkId,
                        },
                        value: {
                            value: expectedValue
                        }
                    });
                    const updateResult = await deepClient.serial({
                        operations: [
                            operation
                        ]
                    });
                    chai_1.assert.equal(updateResult.error, undefined);
                    chai_1.assert.notEqual(updateResult.data, undefined);
                    chai_1.assert.strictEqual(updateResult.data.length, 1);
                    const { data: [newLink] } = await deepClient.select({
                        id: newLinkId
                    });
                    chai_1.assert.strictEqual(newLink.value.value, expectedValue);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
            it('multiple updates in one operation', async () => {
                let linkIdsToDelete = [];
                const expectedValue = 'newStringValue';
                const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
                try {
                    const insertResult = await deepClient.insert({
                        type_id: typeTypeLinkId,
                        string: {
                            data: {
                                value: "stringValue"
                            }
                        }
                    });
                    chai_1.assert.equal(insertResult.error, undefined);
                    chai_1.assert.notEqual(insertResult.data, undefined);
                    chai_1.assert.strictEqual(insertResult.data.length, 1);
                    const newLinkId = insertResult.data[0].id;
                    linkIdsToDelete.push(newLinkId);
                    const operation = (0, gql_1.createSerialOperation)({
                        table: 'strings',
                        type: 'update',
                        exp: {
                            link_id: newLinkId,
                        },
                        value: {
                            value: expectedValue
                        }
                    });
                    const updateResult = await deepClient.serial({
                        operations: [
                            operation,
                            operation
                        ]
                    });
                    chai_1.assert.equal(updateResult.error, undefined);
                    chai_1.assert.notEqual(updateResult.data, undefined);
                    chai_1.assert.strictEqual(updateResult.data.length, 2);
                    const { data: [newLink] } = await deepClient.select({
                        id: newLinkId
                    });
                    chai_1.assert.strictEqual(newLink.value.value, expectedValue);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
            it('multiple updates in multiple operations', async () => {
                let linkIdsToDelete = [];
                const expectedValue = 'newStringValue';
                const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
                try {
                    const insertResult = await deepClient.insert({
                        type_id: typeTypeLinkId,
                        string: {
                            data: {
                                value: "stringValue"
                            }
                        }
                    });
                    chai_1.assert.equal(insertResult.error, undefined);
                    chai_1.assert.notEqual(insertResult.data, undefined);
                    chai_1.assert.strictEqual(insertResult.data.length, 1);
                    const newLinkId = insertResult.data[0].id;
                    linkIdsToDelete.push(newLinkId);
                    const operation = (0, gql_1.createSerialOperation)({
                        table: 'strings',
                        type: 'update',
                        exp: {
                            link_id: newLinkId,
                        },
                        value: {
                            value: expectedValue
                        }
                    });
                    const updateResult = await deepClient.serial({
                        operations: [
                            operation,
                            operation
                        ]
                    });
                    chai_1.assert.equal(updateResult.error, undefined);
                    chai_1.assert.notEqual(updateResult.data, undefined);
                    chai_1.assert.strictEqual(updateResult.data.length, 2);
                    const { data: [newLink] } = await deepClient.select({
                        id: newLinkId
                    });
                    chai_1.assert.strictEqual(newLink.value.value, expectedValue);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
        });
        describe('delete', () => {
            it('one delete', async () => {
                let linkIdsToDelete = [];
                const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
                try {
                    const { data: [{ id: newLinkId }] } = await deepClient.insert({
                        type_id: typeTypeLinkId,
                        string: {
                            data: {
                                value: "stringValue"
                            }
                        }
                    });
                    linkIdsToDelete.push(newLinkId);
                    const operation = (0, gql_1.createSerialOperation)({
                        table: 'links',
                        type: 'delete',
                        exp: {
                            id: newLinkId
                        }
                    });
                    const result = await deepClient.serial({
                        operations: [
                            operation,
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    chai_1.assert.strictEqual(result.data.length, 1);
                    const { data: [newLink] } = await deepClient.select(newLinkId);
                    chai_1.assert.equal(newLink, undefined);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
            it('multiple deletes in one operation', async () => {
                let linkIdsToDelete = [];
                try {
                    const reservedLinkIds = await deepClient.reserve(2);
                    linkIdsToDelete = [...linkIdsToDelete, ...reservedLinkIds];
                    const result = await deepClient.serial({
                        operations: [
                            {
                                table: 'links',
                                type: 'delete',
                                exp: {
                                    id: reservedLinkIds[0]
                                }
                            },
                            {
                                table: 'links',
                                type: 'delete',
                                exp: {
                                    id: reservedLinkIds[1]
                                }
                            }
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    chai_1.assert.strictEqual(result.data.length, 2);
                    const { data } = await deepClient.select(reservedLinkIds);
                    chai_1.assert.equal(data.length, 0);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
            it('multiple deletes in multiple operations', async () => {
                let linkIdsToDelete = [];
                try {
                    const reservedLinkIds = await deepClient.reserve(2);
                    linkIdsToDelete = [...linkIdsToDelete, ...reservedLinkIds];
                    const result = await deepClient.serial({
                        operations: [
                            {
                                table: 'links',
                                type: 'delete',
                                exp: {
                                    id: reservedLinkIds[0]
                                }
                            },
                            {
                                table: 'links',
                                type: 'delete',
                                exp: {
                                    id: reservedLinkIds[1]
                                }
                            }
                        ]
                    });
                    chai_1.assert.equal(result.error, undefined);
                    chai_1.assert.notEqual(result.data, undefined);
                    chai_1.assert.strictEqual(result.data.length, 2);
                    const { data } = await deepClient.select(reservedLinkIds);
                    chai_1.assert.equal(data.length, 0);
                }
                finally {
                    await deepClient.delete(linkIdsToDelete);
                }
            });
        });
    });
    it('select string table', async () => {
        const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
        const { data: [newLink] } = await deepClient.insert({
            type_id: typeTypeLinkId,
            string: {
                data: {
                    value: "stringValue"
                }
            }
        });
        await deepClient.select({
            link_id: { _eq: newLink.id }
        }, {
            table: 'strings'
        });
    });
    it('deepClient token must not be undefined when secret is passed to apolloClient', async () => {
        const apolloClient = (0, client_js_1.generateApolloClient)({
            path: graphQlPath,
            ssl: true,
            ws: true,
            secret
        });
        const deep = new client_1.DeepClient({ apolloClient });
        chai_1.assert.notEqual(deep.token, undefined);
    });
    describe('react', () => {
        describe(`useDeepSubscription`, () => {
            it(`type`, async () => {
                await setup();
                expect(react_1.screen.getByText('Loading...')).toBeInTheDocument();
                await (0, react_1.waitFor)(() => {
                    expect(react_1.screen.queryByText('Loading...')).not.toBeInTheDocument();
                });
                expect(react_1.screen.queryByText(/^Error:/)).not.toBeInTheDocument();
                await (0, react_1.waitFor)(() => {
                    const dataLengthText = react_1.screen.getByText(/items loaded$/);
                    expect(dataLengthText).toBeInTheDocument();
                    const dataLength = parseInt(dataLengthText.textContent);
                    expect(dataLength).toBeGreaterThan(0);
                }, { timeout: 10000 });
                async function setup() {
                    const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
                    function TestHookComponent() {
                        const { loading, data, error } = (0, client_1.useDeepSubscription)({
                            type_id: typeTypeLinkId,
                        });
                        if (loading) {
                            return react_2.default.createElement("div", null, "Loading...");
                        }
                        if (error) {
                            return react_2.default.createElement("div", null,
                                "Error: ",
                                error.message);
                        }
                        return react_2.default.createElement("div", null,
                            data.length,
                            " items loaded");
                    }
                    (0, react_1.render)(react_2.default.createElement(index_js_1.ApolloProvider, { client: deepClient.apolloClient },
                        react_2.default.createElement(client_2.DeepProvider, null,
                            react_2.default.createElement(TestHookComponent, null))));
                }
            });
            it('rerender with loading:true must occur only once', async () => {
                let renderCount = 0;
                let loadingCount = 0;
                function TestComponent() {
                    const deepSubscriptionResult = (0, client_1.useDeepSubscription)({
                        id: {
                            _id: ['@deep-foundation/core']
                        }
                    });
                    renderCount += 1;
                    if (deepSubscriptionResult.loading) {
                        loadingCount += 1;
                    }
                    return null;
                }
                (0, react_1.render)(react_2.default.createElement(index_js_1.ApolloProvider, { client: deepClient.apolloClient },
                    react_2.default.createElement(client_2.DeepProvider, null,
                        react_2.default.createElement(TestComponent, null))));
                await (0, react_1.waitFor)(() => {
                    (0, chai_1.assert)(renderCount > 1, 'TestComponent must be rerendered more than once');
                    (0, chai_1.assert)(loadingCount === 1, 'loading:true must occur only once');
                });
            });
            it('rerender with loading:false must occur only once', async () => {
                let renderCount = 0;
                let loadingCount = 0;
                function TestComponent() {
                    const deepSubscriptionResult = (0, client_1.useDeepSubscription)({
                        id: {
                            _id: ['@deep-foundation/core']
                        }
                    });
                    renderCount += 1;
                    if (!deepSubscriptionResult.loading) {
                        loadingCount += 1;
                    }
                    return null;
                }
                (0, react_1.render)(react_2.default.createElement(index_js_1.ApolloProvider, { client: deepClient.apolloClient },
                    react_2.default.createElement(client_2.DeepProvider, null,
                        react_2.default.createElement(TestComponent, null))));
                await (0, react_1.waitFor)(() => {
                    (0, chai_1.assert)(renderCount > 1, 'TestComponent must be rerendered more than once');
                    (0, chai_1.assert)(loadingCount === 1, 'loading:false must occur only once');
                });
            });
        });
        describe('login', () => {
            it('login with token', async () => {
                const apolloClient = (0, client_js_1.generateApolloClient)({
                    path: graphQlPath,
                    ssl: ssl,
                });
                const unloginedDeep = new client_1.DeepClient({ apolloClient });
                const guest = await unloginedDeep.guest();
                const guestDeep = new client_1.DeepClient({ deep: unloginedDeep, ...guest });
                const admin = await guestDeep.login({
                    linkId: await guestDeep.id('deep', 'admin'),
                });
                const adminToken = admin.token;
                const deep = new client_1.DeepClient({ deep: guestDeep, ...admin });
                let deepInComponent;
                function DeepConsumerComponent() {
                    deepInComponent = (0, client_3.useDeep)();
                    return null;
                }
                (0, react_1.render)(react_2.default.createElement(capacitor_1.CapacitorStoreProvider, null,
                    react_2.default.createElement(query_1.QueryStoreProvider, null,
                        react_2.default.createElement(cookies_1.CookiesStoreProvider, null,
                            react_2.default.createElement(local_1.LocalStoreProvider, null,
                                react_2.default.createElement(react_token_1.TokenProvider, null,
                                    react_2.default.createElement(apollo_client_tokenized_provider_1.ApolloClientTokenizedProvider, { options: {
                                            client: "@deep-foundation/sdk",
                                            token: adminToken,
                                            path: graphQlPath,
                                            ws: true,
                                        } },
                                        react_2.default.createElement(client_2.DeepProvider, null,
                                            react_2.default.createElement(DeepConsumerComponent, null)))))))));
                await (0, react_1.waitFor)(() => {
                    (0, chai_1.assert)(deepInComponent.linkId !== 0, 'deep.linkId is 0. Failed to login');
                });
            });
            it('login with token in apollo client', async () => {
                // await deepClient.whoami(); // ApolloError: Int cannot represent non-integer value: NaN
                const unloginedDeep = new client_1.DeepClient({ apolloClient });
                const guest = await unloginedDeep.guest();
                const guestDeep = new client_1.DeepClient({ deep: unloginedDeep, ...guest });
                const admin = await guestDeep.login({
                    linkId: await guestDeep.id('deep', 'admin'),
                });
                const deepClient = new client_1.DeepClient({ deep: guestDeep, ...admin });
                chai_1.assert.isTrue(!!deepClient.token);
                let deepInComponent;
                function TestComponent() {
                    deepInComponent = (0, client_3.useDeep)();
                    return null;
                }
                (0, react_1.render)(react_2.default.createElement(query_1.QueryStoreProvider, null,
                    react_2.default.createElement(local_1.LocalStoreProvider, null,
                        react_2.default.createElement(apollo_client_tokenized_provider_1.ApolloClientTokenizedProvider, { options: {
                                path: graphQlPath,
                                ssl: ssl,
                                token: deepClient.token
                            } },
                            react_2.default.createElement(react_token_1.TokenProvider, null,
                                react_2.default.createElement(client_2.DeepProvider, null,
                                    react_2.default.createElement(TestComponent, null)))))));
                await (0, react_1.waitFor)(() => {
                    (0, chai_1.assert)(deepInComponent.linkId !== 0, 'deep.linkId is 0. Failed to login');
                });
            });
        });
    });
    describe('short value insert', () => {
        it('insert string', async () => {
            const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
            const { data: [newLink] } = await deepClient.insert({
                type_id: typeTypeLinkId,
                string: 'helloBugFixers'
            });
            const stringRow = await deepClient.select({
                link_id: { _eq: newLink.id }
            }, { table: 'strings' });
            if (newLink?.id)
                deepClient.delete(newLink.id);
            chai_1.assert.equal(stringRow?.data?.[0]?.value, 'helloBugFixers');
        });
        it('insert number', async () => {
            const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
            const { data: [newLink] } = await deepClient.insert({
                type_id: typeTypeLinkId,
                number: 0
            });
            const numberRow = await deepClient.select({
                link_id: { _eq: newLink.id }
            }, { table: 'numbers' });
            if (newLink?.id)
                deepClient.delete(newLink.id);
            chai_1.assert.equal(numberRow?.data?.[0]?.value, 0);
        });
        it('insert object', async () => {
            const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
            const { data: [newLink] } = await deepClient.insert({
                type_id: typeTypeLinkId,
                object: { message: 'helloBugFixers' }
            });
            const objectRow = await deepClient.select({
                link_id: { _eq: newLink.id }
            }, { table: 'objects' });
            if (newLink?.id)
                deepClient.delete(newLink.id);
            chai_1.assert.deepEqual(objectRow?.data?.[0]?.value, { message: 'helloBugFixers' });
        });
        it('insert inherited links with values', async () => {
            const typeTypeLinkId = await deepClient.id("@deep-foundation/core", "Type");
            const { data: [typeLink] } = await deepClient.insert({ type_id: typeTypeLinkId });
            const { data: [newLink] } = await deepClient.insert({
                type_id: typeTypeLinkId,
                string: 'helloBugFixers',
                from: {
                    type_id: typeTypeLinkId,
                    number: 0,
                    in: {
                        type_id: typeTypeLinkId,
                        from_id: typeLink.id,
                        number: 1
                    }
                },
                to: {
                    type_id: typeTypeLinkId,
                    object: { string: 'goodbyeBugFixers' },
                    out: {
                        type_id: typeTypeLinkId,
                        to_id: typeLink.id,
                        number: 2
                    }
                }
            });
            const { data: [selected] } = await deepClient.select(newLink.id);
            const { data: [selectedFrom] } = await deepClient.select(selected.from_id);
            const { data: [selectedTo] } = await deepClient.select(selected.to_id);
            const { data: [selectedIn] } = await deepClient.select({ to_id: selected.from_id });
            const { data: [selectedOut] } = await deepClient.select({ from_id: selected.to_id });
            if (selected?.id)
                deepClient.delete(selected.id);
            if (selected?.from_id)
                deepClient.delete(selected.from_id);
            if (selectedIn?.id)
                deepClient.delete(selectedIn.id);
            if (selectedOut?.id)
                deepClient.delete(selectedOut.id);
            if (selectedOut?.id)
                deepClient.delete(typeLink.id);
            chai_1.assert.equal(selected?.value?.value, 'helloBugFixers');
            chai_1.assert.equal(selectedFrom?.value?.value, 0);
            chai_1.assert.equal(selectedTo?.value?.value?.string, 'goodbyeBugFixers');
            chai_1.assert.equal(selectedIn?.value?.value, 1);
            chai_1.assert.equal(selectedOut?.value?.value, 2);
        });
    });
});
//# sourceMappingURL=client.js.map