"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const client_js_1 = require("@deep-foundation/hasura/client.js");
const api_js_1 = require("@deep-foundation/hasura/api.js");
const client_1 = require("../imports/client");
const chai_1 = require("chai");
const handlers_1 = require("../imports/handlers");
const promise_1 = require("../imports/promise");
exports.api = new api_js_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const apolloClient = (0, client_js_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_1.DeepClient({ apolloClient });
describe('tree', () => {
    it(`promiseTree`, async () => {
        let linksToDelete = [];
        const typeTypeLinkId = await deep.id("@deep-foundation/core", "Type");
        const handleInsertTypeLinkId = await deep.id("@deep-foundation/core", "HandleInsert");
        const { data: [{ id: customTypeLinkId }] } = await deep.insert({
            type_id: typeTypeLinkId,
        });
        linksToDelete.push(customTypeLinkId);
        const handler = await (0, handlers_1.insertHandler)(handleInsertTypeLinkId, customTypeLinkId, `async () => {}`);
        const { data: [{ id: customLinkId }] } = await deep.insert({
            type_id: customTypeLinkId
        });
        linksToDelete.push(customLinkId);
        await (0, promise_1.delay)(5000);
        try {
            const { data: promiseTreeLinksDownToCustomLink } = await deep.select({
                up: {
                    parent_id: { _eq: customLinkId },
                    tree_id: { _eq: await deep.id("@deep-foundation/core", "promiseTree") }
                }
            });
            console.log({ promiseTreeLinksDownToCustomLink });
            linksToDelete = [...linksToDelete, ...promiseTreeLinksDownToCustomLink.map(link => link.id)];
            const thenTypeLinkId = await deep.id("@deep-foundation/core", "Then");
            const thenLinkId = promiseTreeLinksDownToCustomLink.find(link => link.type_id === thenTypeLinkId);
            chai_1.assert.notEqual(thenLinkId, undefined);
            const promiseTypeLinkId = await deep.id("@deep-foundation/core", "Promise");
            const promiseLinkId = promiseTreeLinksDownToCustomLink.find(link => link.type_id === promiseTypeLinkId);
            chai_1.assert.notEqual(promiseLinkId, undefined);
            const resolvedTypeLinkId = await deep.id("@deep-foundation/core", "Resolved");
            const resolvedLinkId = promiseTreeLinksDownToCustomLink.find(link => link.type_id === resolvedTypeLinkId);
            chai_1.assert.notEqual(resolvedLinkId, undefined);
        }
        finally {
            await (0, handlers_1.deleteHandler)(handler);
            await deep.delete(linksToDelete);
        }
    });
});
//# sourceMappingURL=tree.js.map